/*------------------------------------------------------------------------
	File		    : abl2db_launcher.p
	Purpose	    : Invokes startup classes and initiates process
	Syntax	    : Run abl2db_launcher.p
	Description : Top level procedure for ABL2DB application	
	Author(s)   : Thomas Mercer-Hursh, Ph.D.
	Created	    : Wed Feb 26 13:48:00 CST 2014
	Notes	      :
  ----------------------------------------------------------------------*/
using Progress.Lang.*.
using com.cintegrity.ABL2DB.Driver.

block-level on error undo, throw.

/* ***************** Application Specific Definitions ***************** */
/* Variables for selecting files from source directories
Compilable is all compilable
Source is all source, e.g., including include
*/
define variable chCompilableExtensions as character no-undo initial "p,w,cls".
define variable chSourceExtensions as character no-undo initial "bat,cls,i,inq,f,p,pn,po,px,sh,v".
define variable lgExcludeNoExtension as logical no-undo initial false.
define variable chExcludeDirectories as character no-undo initial "".
define variable lgIncludeEmptyDirectories as logical no-undo.
/* Other lists which can be useful in testing
NonSource is something that may validly be in the source tree, but which is not source.
Junk is something currently in source tree that shouldn't be
define variable chNonSourceExt as character no-undo initial "cl,d,i_d,p_d,p_m,s,t,td,tq,tu".
define variable chJunkExt as character no-undo initial "a,bak,frm,io,ldf,lst,ml,new,note,o,old,pcap,p_m_s,p_s,r,sv,s2,tab,txt,zip,1,2,$75,$77".
*/
/* Database & schema variables */
define variable chDataBaseList as character no-undo initial "c:\work\IS\DB\ISRel,c:\work\IS\DB\ISSort".
define variable chLogicalDataBaseList as character no-undo initial "ISRel,ISSort,DICTDB".
define variable chAliasList as character no-undo initial "".
define variable chDataBaseConnectParams as character no-undo initial "-1|-1".
define variable chSchemaDirectory as character no-undo initial "c:\work\IS\DB".
define variable chSchemaFileList as character no-undo initial "ISrel,ISsort,DICTDB".
/* Proparse schema file is proparse.schema in chSchemaDirectory */
/* Propath needed for compile */
define variable chPropath as character no-undo initial "c:\work\IS\Joe\src,c:\work\IS\ISrel\run,c:\work\IS\ISrel\src".
/* Source Variables */
define variable chSourceDirectoryList as character no-undo initial "c:\work\IS\Joe\src,c:\work\IS\ISrel\src".
define variable chRunDirectoryList as character no-undo initial "c:\work\IS\Joe\run,c:\work\IS\ISrel\run".
define variable chXrefDirectoryList as character no-undo initial "c:\work\IS\Joe\xrf,c:\work\IS\ISrel\xrf".
define variable chListDirectoryList as character no-undo initial "c:\work\IS\Joe\lst,c:\work\IS\ISrel\lst".
define variable chDebugDirectoryList as character no-undo initial "c:\work\IS\Joe\debug,c:\work\IS\ISrel\debug".
/* Other variables */ 
define variable chReportDestination as character no-undo initial "c:\work\IS\ISrel\rpt".
define variable chMenuFile as character no-undo initial "c:\work\IS\DB\Menu.d".
define variable chMenuItemFile as character no-undo initial "c:\work\IS\DB\MenuItem.d".
define variable chFunctionalUnitFile as character initial "c:\work\IS\DB\FunctionalUnit.d".
define variable chPrimaryLog as character initial "c:\work\ABL2UML\ABL2DB\rpt\ABL2DB.lg".
define variable inWhich as integer no-undo.
      
/* ************************ Global Definitions ************************ */
define variable obDriver as class Driver no-undo.

/* ***************************  Main Block  *************************** */
do on stop undo, retry:
	if retry
	then do:
		log-manager:write-message ("STOP condition encountered", "OtherERROR").
		undo, leave.
  	end.

	do on error undo, leave:
		session:error-stack-trace = true.
		log-manager:logfile-name = chPrimaryLog.
		log-manager:clear-log ().
		/* Control log detail and event types */
		log-manager:logging-level = 3.
		/* Uncomment for detailed trace during debugging */
		/* log-manager:log-entry-types = "DynObjects.Class,4GLTrace". */ 
		log-manager:write-message ("Start ABL2DB").

		/* ************************ Begin Actual Work ********************* */
    obDriver = new Driver().
    assign
        obDriver:chDataBaseList            = chDataBaseList
        obDriver:chLogicalDataBaseList     = chLogicalDataBaseList
        obDriver:chAliasList               = chAliasList
        obDriver:ChDataBaseConnectParams   = chDataBaseConnectParams
        obDriver:chSchemaDirectory         = chSchemaDirectory
        obDriver:chSchemaFileList          = chSchemaFileList  
        obDriver:chSourceDirectoryList     = chSourceDirectoryList
        obDriver:chRunDirectoryList        = chRunDirectoryList   
        obDriver:chListDirectoryList       = chListDirectoryList  
        obDriver:chXrefDirectoryList       = chXrefDirectoryList  
        obDriver:chDebugDirectoryList      = chDebugDirectoryList 
        obDriver:chReportDestination       = chReportDestination
        obDriver:chPropath                 = chPropath          
        obDriver:chCompilableExtensions    = chCompilableExtensions
        obDriver:chSourceExtensions        = chSourceExtensions
        obDriver:lgExcludeNoExtension      = lgExcludeNoExtension
        obDriver:chExcludeDirectories      = chExcludeDirectories
        obDriver:lgIncludeEmptyDirectories = lgIncludeEmptyDirectories
        obDriver:chMenuFile                = chMenuFile
        obDriver:chMenuItemFile            = chMenuItemFile
        obDriver:chFunctionalUnitFile      = chFunctionalUnitFile
        .
    obDriver:Initialize().
    obDriver:Process().
 
		/* Cleanup */

		if valid-object( obDriver ) then delete object obDriver.

		/* ************************ End Actual Work ********************* */
	
		catch eobAppError as Progress.Lang.AppError:
			log-manager:write-message ( eobAppError:ReturnValue ).	
			delete object eobAppError.
		end catch.
	
		catch eobSysError as Progress.Lang.SysError :
			log-manager:write-message(substitute("Unexpected Exception: &1", eobSysError:getMessage(1)), "SysERROR").
			if eobSysError:NumMessages > 1
			then do inWhich = 2 to eobSysError:NumMessages:
				log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SysERROR").
			end.
			do inWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
				log-manager:write-message(entry(inWhich,eobSysError:CallStack,"~n"),"CALLSTACK").
			end.
			delete object eobSysError no-error.
		end catch.
		
    catch eobOtherError as Progress.Lang.Error :
      log-manager:write-message(substitute("Other error: &2", eobOtherError:getMessage(1))).  
      if eobOtherError:NumMessages > 1
      then do inWhich = 2 to eobOtherError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobOtherError:getMessage(inWhich)), "OtherERROR").
      end.
      do inWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        log-manager:write-message(entry(inWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobOtherError no-error.
    end catch.

		finally:
			log-manager:write-message ("End ABL2DB").
		end finally.

	end. /* do */
  
end.
