 /*------------------------------------------------------------------------
    File        : BuildBlocks
    Purpose     : Scan code base and build objects for procedures, classes, IPs, methods, functions, etc.
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 15 14:51:48 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildBlocksOne.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildBlocks inherits BuildDirectoryTree: 
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chSchemaDirectory as character no-undo get. set.
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalBlocks as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.

  define private variable obBuildBlocksOne as class BuildBlocksOne no-undo.
  
       /* Note: originally, this table included line number, but we are currently not
          reporting on duplicates, so this has been eliminated pending conversion to
          tracking them by scope. */
  define private temp-table ttBufferMatch
      field chBufferName as character
      field chDataBaseName as character
      field chDataBaseID as character
      field chTableName as character
      field chTableID as character
      field lgIsDBTable as logical
      field chFileName as character
      index ixByBuffer as unique primary chBufferName
      .

 	constructor public BuildBlocks ( 
      ipchSourceDirectoryectory as character
      ):  
    super ( ipchSourceDirectoryectory ).
	end constructor.

	destructor public BuildBlocks ( ):
	  if valid-object(obReport) 
    then obReport:WriteMessage ( substitute("End of BuildBlocks with &1 total blocks in &2 compile units of &3 total files",inTotalBlocks, inTotalCompiled, inTotalFiles ) ).
    log-manager:write-message ( substitute( "End BuildBlocks with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    super:Initialize().

    log-manager:write-message ("Start BuildBlocks").
    obReport:WriteMessage ( substitute("Exception list for object build of &1", chSourceDirectoryList ) ).

  end method.
  
  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll ().
    
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not lgIsDir:
      obBuildBlocksOne = new BuildBlocksOne( obReport ).
      obBuildBlocksOne:chLogicalDataBaseList = chLogicalDataBaseList.
      obBuildBlocksOne:chAliasList = chAliasList.
      obBuildBlocksOne:chSchemaDirectory = chSchemaDirectory.
      obBuildBlocksOne:Initialize().
      obBuildBlocksOne:ProcessOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      inTotalFiles = inTotalFiles + obBuildBlocksOne:inTotalFiles.
      inTotalCompiled = inTotalCompiled + obBuildBlocksOne:inTotalCompiled.
      inTotalExceptions = inTotalExceptions + obBuildBlocksOne:inTotalExceptions.
      inTotalBlocks = inTotalBlocks + obBuildBlocksOne:inTotalBlocks.
      bfContents.lgIsProcessed = true.
      
      delete object obBuildBlocksOne.
    end. /* for each */
  end method.

end class.