 /*------------------------------------------------------------------------
    File        : BuildRunLinks
    Purpose     : Scan code base and build objects for procedures, classes, IPs, methods, functions, etc.
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 15 14:51:48 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildRunLinks inherits BuildDirectoryTree: 
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.
  
  define private temp-table ttPersistentProc no-undo
      field chCompileUnitID as character
      index ixByID as primary unique chCompileUnitID.
      .
  define private variable hnXrefHandler as handle no-undo.
  define private variable chCompileUnitID as character no-undo.
  define private variable chDiskFileID as character no-undo.
  define private variable chCompileUnitName as character no-undo.
  define private variable chSourceUnitFullPath as character no-undo.
  define private variable chCurrentReferenceType as character no-undo.
  define private variable chCurrentCompileSubUnitID as character no-undo.
  define private variable chCurrentCompileSubUnitName as character no-undo.
  define private variable chCurrentIncludeReferenceID as character no-undo.
  define private variable chIncludeSourcePath as character no-undo.
  define private variable chCurrentCURunTargetID as character no-undo.
  define private variable chCurrentCSURunTargetID as character no-undo.
  define private variable chCurrentNodeType as character no-undo.
  define private variable chCurrentModeValue as character no-undo.
  define private variable chContext as character no-undo.
  define private variable chCurrentProgramCallID as character no-undo.
  define private variable chLineNo as character no-undo.
  define private variable inTotalExistingCU as integer no-undo.
  define private variable inTotalCSUinThisCU as integer no-undo.
  define private variable inTotalCSUinPP as integer no-undo.  
  define private variable inTotalUnresolved as integer no-undo.
  
	constructor public BuildRunLinks ( 
      ipchSourceDirectoryectory as character
      ):  
    super ( ipchSourceDirectoryectory ).
	end constructor.

	destructor public BuildRunLinks ( ):
    if valid-object(obReport) 
    then obReport:WriteMessage  ( substitute("End of scan with &1 total files (&2 compiled) with &3 Existing CU, &4 CSU in this CU, &5 CSU in PP, and &6 unresolved",inTotalFiles, inTotalCompiled, inTotalExistingCU, inTotalCSUinThisCu, inTotalCSUinPP, inTotalUnresolved ) ).
	  log-manager:write-message ( substitute( "End of BuildRunLinks with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    super:Initialize().
 
    log-manager:write-message ( "Start of BuildRunLinks").
    obReport:WriteMessage  ( substitute("Exception list for BuildRunLinks of &1", chSourceDirectoryList ) ).

    run com/cintegrity/ABL2DB/RunLinkXrefHandler.p persistent set hnXrefHandler.
    run Initialize in hnXrefHandler ( this-object ).
  end method.
  
  method public void CleanUp():
    if valid-handle(hnXrefHandler)
    then do:
      delete procedure hnXrefHandler.
      hnXrefHandler = ?.
    end.
    else log-manager:write-message ("BuildRunLinks: Destructor: Invalid handle for hnXrefHandler").
  end.
  
  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll ().
    
    CleanUp().
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not bfContents.lgIsDir:
      ScanOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.

  method private void ScanOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchXrefFile as character no-undo.

    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
    
    inTotalFiles = inTotalFiles + 1.

    do for bfDiskFile:
      find bfDiskFile no-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if not available ( bfDiskFile )
      then do:
        obReport:WriteMessage  ( substitute("BuildRunLinks: DiskFile missing for &1", ipchFullPath ) ). 
        inTotalExceptions = inTotalExceptions + 1.
        return.
      end.
      if bfDiskFile.lgCompilable
      then do for bfCompileUnit:
        find bfCompileUnit no-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
        if available ( bfCompileUnit )
        then do:
          assign
              inTotalCompiled = inTotalCompiled + 1
              mchXrefFile = obPathUtilities:XrefFile( bfCompileUnit.chXrefBaseDir, ipchPackage, ipchName )
              chCompileUnitName = obPathUtilities:RelativePathFromComponents( ipchPackage, ipchName )
              chCompileUnitID = bfCompileUnit.chID.
              .
          empty temp-table ttPersistentProc.
          ProcessXref ( mchXrefFile ).
        end. /* available bfCompileUnit */
        else do:
         obReport:WriteMessage  ( substitute("BuildSubLinks: CompileUnit missing for &1", ipchFullPath ) ). 
         inTotalExceptions = inTotalExceptions + 1.
         return.
        end.
      end. /* lgCompilable */
    end. /* for bfDiskFile */
  end method.
  
  method private void ProcessXref ( 
      ipchXrefTarget as character
      ):
    define variable mhnParser as handle no-undo.
  
    file-info:file-name = ipchXrefTarget.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      obReport:WriteMessage ( substitute("BuildRunLinks: XREF file '&1' not found or not readable", ipchXrefTarget ) ).
      inTotalExceptions = inTotalExceptions + 1.
      return.
    end.     
 
    do on error undo, leave:
      create sax-reader mhnParser.
      mhnParser:handler = hnXrefHandler.
      mhnParser:set-input-source("file", ipchXrefTarget ).
      mhnParser:sax-parse() no-error.
      if error-status:error
        then do:
          if error-status:num-messages gt 0
          then log-manager:write-message( error-status:get-message(1)).
          else log-manager:write-message( return-value ).
        end.
      delete object mhnParser.
      mhnParser  = ?.      
      catch eobAppError as Progress.Lang.AppError:
        log-manager:write-message ( eobAppError:ReturnValue ).  
        delete object eobAppError.
        if valid-handle (mhnParser) then delete object mhnParser.
        mhnParser  = ?.
      end catch.  
    end.
  end method.

  method public void StartElement (
      ipchNameSpaceURI as character,
      ipchLocalName as character,
      ipchElementType as character,
      iphnAttributes as handle
      ):
        
    define variable mchIdentifier as character no-undo.
    define variable mchName as character no-undo.
    define variable mchPackage as character no-undo.
    define variable mchBaseName as character no-undo.
    define variable mchSourceDirectory as character no-undo.
      
    define buffer bfCompileUnit for CompileUnit.
    define buffer bfCompileSubUnit for CompileSubUnit.
    define buffer bfIncludeUnit for IncludeUnit.
    define buffer bfIncludeReference for IncludeReference.
    define buffer bfDiskFile for DiskFile.
    define buffer bfProgramCall for ProgramCall.
      
    case ipchElementType:
      when "REFERENCE"                           /* All major nodes are a reference */
        then do:
          chCurrentReferenceType = iphnAttributes:get-value-by-qname("Reference-type").
          mchIdentifier = replace(trim(iphnAttributes:get-value-by-qname("Object-identifier")), chAntiSeparator, chSeparator).
          case chCurrentReferenceType:
            when "RUN"                                 /* run program or subprogram */ 
              then do:
                /* Need to search for run target - CU, CSU in this CU, CSU in PP, or unresolved */
                assign 
                    chCurrentProgramCallID = ""
                    mchSourceDirectory = obPathUtilities:SourceDirectoryFromRelativePath ( mchIdentifier ).
                    mchPackage = obPathUtilities:PackageFromRelativePath ( mchIdentifier ).
                    mchName = obPathUtilities:NameFromPath ( mchIdentifier ).
                    .
              
                /* Check if Run Target is existing CompileUnit */
                do for bfDiskFile:
                  find bfDiskFile no-lock 
                      where bfDiskFile.chBaseDirectory = mchSourceDirectory
                      and   bfDiskFile.chPackage = mchPackage 
                      and bfDiskFile.chName = mchName 
                      no-error.
                  if available( bfDiskFile )
                  then do for bfCompileUnit:
                    find bfCompileUnit no-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
                    if available( bfCompileUnit )
                    then do for bfProgramCall:
                      create bfProgramCall.
                      assign
                          bfProgramCall.chID = guid
                          chCurrentProgramCallID = bfProgramCall.chID
                          bfProgramCall.chCallingUnitID = chCompileUnitID
                          bfProgramCall.chCallingSubUnitID = ""
                          bfProgramCall.chCalledUnitID = bfCompileUnit.chID
                          bfProgramCall.chCalledSubUnitID = ""
                          bfProgramCall.lgDynamic = no
                          bfProgramCall.lgResolved = yes
                          inTotalExistingCU = inTotalExistingCU + 1
                          .
                    end. 
                  end. /* available bfDiskFile */
                end. /* of for DiskFile */
                /* If not yet resolved, check if Run Target is CSU in this CU */                
                if chCurrentProgramCallID = ""
                then do:
                  for each bfCompileSubUnit no-lock where bfCompileSubUnit.chCompileUnitID = chCompileUnitID:
                    if bfCompileSubUnit.chName = mchIdentifier
                    then do for bfProgramCall:
                      create bfProgramCall.
                      assign
                          bfProgramCall.chID = guid
                          chCurrentProgramCallID = bfProgramCall.chID
                          bfProgramCall.chCallingUnitID = chCompileUnitID
                          bfProgramCall.chCallingSubUnitID = ""
                          bfProgramCall.chCalledUnitID = chCompileUnitID
                          bfProgramCall.chCalledSubUnitID = bfCompileSubUnit.chID
                          bfProgramCall.lgDynamic = no
                          bfProgramCall.lgResolved = yes
                          inTotalCSUinThisCU = inTotalCSUinThisCU + 1
                          .
                    end.
                  end.
                end. /* of CSU in this CU */
                /* If not yet resolved, check if Run Target is CSU in a PP */                 
                if chCurrentProgramCallID = ""
                then do:
                  /* Check if Run Target is CSU in current PPs */
                  for each ttPersistentProc no-lock:
                    for each bfCompileSubUnit no-lock where bfCompileSubUnit.chCompileUnitID = ttPersistentProc.chCompileUnitID:
                      if bfCompileSubUnit.chName = mchIdentifier
                      then do for bfProgramCall:
                        create bfProgramCall.
                        assign
                            bfProgramCall.chID = guid
                            chCurrentProgramCallID = bfProgramCall.chID
                            bfProgramCall.chCallingUnitID = chCompileUnitID
                            bfProgramCall.chCallingSubUnitID = ""
                            bfProgramCall.chCalledUnitID = chCompileUnitID
                            bfProgramCall.chCalledSubUnitID = bfCompileSubUnit.chID
                            bfProgramCall.lgDynamic = no
                            bfProgramCall.lgResolved = yes
                            inTotalCSUinPP = inTotalCSUinPP + 1
                            .
                      end.
                    end. /* of CSU in PP */
                  end. /* of for each PersistentProc */
                end.  /* of CSU in PP */ 
                /* If not yet resolved, call is unresolved */                
                if chCurrentProgramCallID = ""
                then do:
                  do for bfProgramCall:
                    create bfProgramCall.
                    assign
                        bfProgramCall.chID = guid
                        chCurrentProgramCallID = bfProgramCall.chID
                        bfProgramCall.chCallingUnitID = chCompileUnitID
                        bfProgramCall.chCallingSubUnitID = ""
                        bfProgramCall.chCalledUnitID = ""
                        bfProgramCall.chCalledSubUnitID = ""
                        bfProgramCall.lgDynamic = no
                        bfProgramCall.lgResolved = yes
                        bfProgramCall.chSourceLine = mchIdentifier
                        inTotalUnresolved = inTotalUnresolved + 1
                        .
                  end.
                end.  
              end.
          end case. /* ReferenceType */
        end. /* REFERENCE */
      when "Line-num"
        then do:
          chCurrentNodeType = "Line-num".
          chLineNo = "".
        end.
      when "Object-context"
        then do:
          chCurrentNodeType = "Object-context".
          chContext = "".
        end.
      otherwise do:
        chCurrentNodeType = "".
      end.
    end. /* case ipchElementType */
  end method.

  method public void Characters (
      ipchData as memptr,
      ipinNumChars as integer
      ):

    case chCurrentNodeType:
      when "Line-num" then chLineNo = chLineNo + GET-STRING(ipchData, 1, get-size(ipchData)).
      when "Object-context" then chContext = chContext + GET-STRING(ipchData, 1, get-size(ipchData)).
    end case.
  end method.

  method public void EndElement (
      ipchNameSpaceURI as character,  
      ipchLocalName as character,
      ipchElementType as character
      ):
    define buffer mbfCompileSubUnit for CompileSubUnit.
    define buffer mbfIncludeReference for IncludeReference.
    define buffer mbfProgramCall for ProgramCall.
    define buffer mbfttPersistentProc for ttPersistentProc.
    
    case ipchElementType:
      when "REFERENCE"                           /* All major nodes are a reference */
        then do:
          case chCurrentReferenceType:
            when "RUN"
              then do:
                UpdateLineNo().
                /* UpdateSourceLine(). See below */
                if chContext = "PERSISTENT"
                then do for mbfProgramCall:
                  find mbfProgramCall exclusive-lock where mbfProgramCall.chID = chCurrentProgramCallID no-error.
                  if not available mbfProgramCall
                  then do:
                    obReport:WriteMessage  ( substitute("LinkXrefHandler.p: ProgramCall not found in Run for &1", chCompileUnitName) ).
                    inTotalExceptions = inTotalExceptions + 1.
                    /* undo, throw new Progress.Lang.AppError( "LinkXrefHandler.p: Aborting." ).   Non-fatal for now */   
                  end.
                  else do for mbfttPersistentProc:
                    mbfProgramCall.lgPersistent = yes.
                    create mbfttPersistentProc.
                    mbfttPersistentProc.chCompileUnitID = mbfProgramCall.chCalledUnitID.
                  end.   
                end.
              end.
          end case. /* chCurrentReferenceType */  
        end. /* REFERENCE */
      otherwise do:
        /* Nothing */
      end.
    end case. /* ElementTYpe */        
  end method.

  method private void UpdateLineNo ():
    define variable minLineNo as integer no-undo.
    define buffer mbfProgramCall for ProgramCall.
    
    do for mbfProgramCall:
      find mbfProgramCall exclusive-lock where mbfProgramCall.chID = chCurrentProgramCallID no-error.
      if not available( mbfProgramCall )
      then do:
        obReport:WriteMessage  ( substitute("LinkXrefHandler.p: ProgramCall not found in UpdateLineNo for &1", chCompileUnitName) ).
        inTotalExceptions = inTotalExceptions + 1.
      end.
      else do:
        assign minLineNo = integer(trim(chLineNo)) no-error.
        if error-status:error
        then do:
          obReport:WriteMessage  ( substitute("LinkXrefHandler.p: Error converting line no = '&1' in UpdateLineNo for &2", chLineNo, chCompileUnitName ) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
        else mbfProgramCall.inLineNo = minLineNo.      
      end. 
    end.
  end method.
      
/* Leave out for now on the theory that the identifier is a better clue for dynamic than the source line
  method private void UpdateSourceLine ():
    define variable mchSourceFile as character no-undo.
    define variable mchALine as character no-undo.
    define variable mlcFile as longchar no-undo.
    define buffer mbfProgramCall for ProgramCall.
    
    do for mbfProgramCall:
      find mbfProgramCall exclusive-lock where mbfProgramCall.chID = chCurrentProgramCallID no-error.
      if not available( mbfProgramCall )
      then obReport:WriteMessage  ( substitute("LinkXrefHandler.p: ProgramCall not found in UpdateSourceLine for &1", chCompileUnitName) ).
      else do:
        copy-lob from file chSourceUnitFullPath to mlcFile.
        if mbfProgramCall.inLineNo = 0 or mbfProgramCall.inLineNo > num-entries( mlcFile, "~n") 
        then return.
        else do:
          mchALine = entry( mbfProgramCall.inLineNo, mlcFile, "~n").
          mbfProgramCall.chSourceLine = mchALine.
        end.
      end. 
    end.
  end method.
*/  
end class.