 /*------------------------------------------------------------------------
    File        : BuildSharedOne
    Purpose     : Build shared variable referesnce for one compile unit
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 21 14:51:48 CDT 2015
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.SchemaUtilities.
using com.cintegrity.FileSystem.PathUtilities.
using com.cintegrity.LoggingUtils.FlyWeightLogger.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.core.TreeUtils from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.
using org.prorefactor.treeparser.Symbol from assembly.
using org.prorefactor.treeparser.SymbolScope from assembly.
using org.prorefactor.treeparser.TableBuffer from assembly.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildSharedOne: 
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chSchemaDirectory as character no-undo get. set.
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalShared as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.
  
  define private variable chCurrentCompileUnitName as character.
  define private variable chFullPath as character no-undo.
  define private variable inCurrentLine as integer no-undo.
  define private variable obSetEnvironment as class SetEnvironment no-undo.
  define private variable obSetSchemaFromFile as class SetSchemaFromFile no-undo.
  define private variable obJavaFile as class java.io.File.
  define private variable obParseUnit as class ParseUnit.
  define private variable obSchemaUtilities as class SchemaUtilities no-undo.
  define private variable obReport as class FlyWeightLogger no-undo. 
  define private variable obPathUtilities as class PathUtilities no-undo.
    
  define temp-table ttField no-undo
      field inOrder as integer
      field chName as character
      field chDataType as character
      field chLike as character
      index ixByOrder as primary unique inOrder
      index ixByName as unique chName
      .
        
 	constructor public BuildSharedOne ( 
      ipobReport as class FlyWeightLogger
      ): 
    obReport = ipobReport.
	end constructor.

	destructor public BuildSharedOne ( ):
    empty temp-table ttField.
    if valid-object(obSchemaUtilities) then delete object obSchemaUtilities.
    if valid-object(obSetEnvironment) then delete object obSetEnvironment.
    if valid-object(obSetSchemaFromFile) then delete object obSetSchemaFromFile.
	end destructor.

  method public void Initialize():
                                                          /* Initialize for Proparse */
    obSetEnvironment = new SetEnvironment().
    obSetEnvironment:Initialize().

    obSetSchemaFromFile = new SetSchemaFromFile().
    obSetSchemaFromFile:chSchemaDirectory = chSchemaDirectory.
    obSetSchemaFromFile:chLogicalDataBaseList = chLogicalDataBaseList.           
    obSetSchemaFromFile:chAliasList = chAliasList.
    obSetSchemaFromFile:lgDeletePPSchemaFile = false.
    obSetSchemaFromFile:Initialize().

    obPathUtilities = new PathUtilities( ).
    obPathUtilities:Initialize().
    
    obSchemaUtilities = new SchemaUtilities ().
    
    catch eobAppError as Progress.Lang.AppError:
      log-manager:write-message ( eobAppError:ReturnValue ).  
      delete object eobAppError.
    end catch.
  end method.
  
  method public void ProcessOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchListFile as character no-undo.
    define variable mchCompileUnitID as character no-undo.
    define variable mchCurrentFullPath as character no-undo.
    define variable mchMangledName as character no-undo.
    
    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
    
    inTotalFiles = inTotalFiles + 1.

    do for bfDiskFile:
      find bfDiskFile no-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if not available ( bfDiskFile )
      then do:
        obReport:WriteMessage ( substitute("BuildSharedOne: DiskFile at &1 not found in ScanOne", ipchFullPath )).
        inTotalExceptions = inTotalExceptions + 1.
        return.  
      end.     
      else do for bfCompileUnit:
        if bfDiskFile.lgCompilable = no then return.      
        find bfCompileUnit no-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
        if available ( bfCompileUnit )
        then do:
          assign
              inTotalCompiled = inTotalCompiled + 1
              chFullPath = ipchFullPath
              chCurrentCompileUnitName = obPathUtilities:FullPathFromComponents( bfDiskFile.chBaseDirectory, bfDiskFile.chPackage, bfDiskFile.chName ).
              .
            ProcessShared ( bfCompileUnit.chID ).
        end. /* available bfCompileUnit */
      end. /* else do */
    end. /* for bfDiskFile */
  end method.
  
  method private void ProcessShared (
      ipchCUID as character
      ):
    do on stop undo, retry:
      if retry
      then do:
        log-manager:write-message ("STOP condition encountered in BuildSharedOne", "SYSERROR").
        return.
      end.
    
      do on error undo, leave:    
        obJavaFile = new java.io.File(chFullPath).
        obParseUnit = new ParseUnit(obJavaFile).
        obParseUnit:treeParser01().       
        
        TreeWalker(obParseUnit:getTopNode(), ipchCUID ).
    
        catch eobAppError as Progress.Lang.AppError:
          log-manager:write-message ( eobAppError:ReturnValue ).  
          delete object eobAppError.
        end catch.
      
        catch eobSysError as Progress.Lang.SysError :
          define variable inWhich as integer no-undo.
          log-manager:write-message(substitute("Unexpected Exception in &1 line &2: &3", chCurrentCompileUnitName, inCurrentLine, eobSysError:getMessage(1)), "SYSERROR").
          if eobSysError:NumMessages > 1
          then do inWhich = 2 to eobSysError:NumMessages:
            log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SYSERROR").
          end.
          do inWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
            log-manager:write-message(entry(inWhich,eobSysError:CallStack,"~n"),"CALLSTACK").
          end.
          delete object eobSysError no-error.
        end catch.
        
        catch eobOtherError as Progress.Lang.Error :
          log-manager:write-message(substitute("Other error in &1 line &2: &3", chCurrentCompileUnitName, inCurrentLine, eobOtherError:getMessage(1))).
          if eobOtherError:NumMessages > 1
          then do inWhich = 2 to eobOtherError:NumMessages:
            log-manager:write-message(substitute("Other error (cont.): &1", eobOtherError:getMessage(inWhich)), "OtherERROR").
          end.
          do inWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
            log-manager:write-message(entry(inWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
          end.
          delete object eobOtherError no-error. 
        end catch.
      end. /* do on error */
    end. /* do on stop */

  end method.

  method private void TreeWalker(
      ipobNode as class JPNode,
      ipchCUID as character
      ):
    if (ipobNode eq ?) then return.
    inCurrentLine = ipobNode:getLine().
    ExamineNode( ipobNode,       ipchCUID ).
    TreeWalker( ipobNode:firstChild(), ipchCUID ).
    TreeWalker( ipobNode:nextSibling(), ipchCUID ).
  end method.
    
  method private void ExamineNode(
      ipobNode as JPNode,
      ipchCUID as character
      ):
    if (not ipobNode:isStateHead()) then return.
    if (ipobNode:getType() eq ProParserTokenTypes:define ) then CollectDefineReference( ipobNode, ipchCUID ).
  end method.
  
  method private void CollectDefineReference(
      ipobNode as class JPNode,
      ipchCUID as character
      ):
    define variable mchDataType as character no-undo.
    define variable mchDBID as character extent 10 no-undo.
    define variable mchDescription as character no-undo.
    define variable mchLike as character no-undo.
    define variable mchName as character no-undo.
    define variable mchObjectType as character no-undo.
    define variable mchRecordName as character extent 10 no-undo.
    define variable mchFieldPhrase as character extent 10 no-undo.
    define variable mchQueryName as character no-undo.
    define variable minNumRecords as integer no-undo.
    define variable mchTableName as character extent 10 no-undo.
    define variable mchAdditionalInfo as character no-undo.
    define variable minStoreType as integer no-undo.
    define variable mchStoreType as character extent 10 no-undo.
    define variable mchBufferName as character no-undo.
    define variable mobBufferSymbol as class Symbol no-undo.
    define variable mobTableBuffer as class TableBuffer no-undo.
    define variable mobTable as class org.prorefactor.core.schema.Table no-undo.
    define variable mchSubTokenType as character no-undo.
    define variable mchTableID as character extent 10 no-undo.
    define variable mchToken as character no-undo.
    define variable mchTokenType as character no-undo.
    define variable minOrder as integer no-undo initial 0.
    define variable minTokenType as integer no-undo.
    define variable mlgIsDBTable as logical extent 10 no-undo initial false.
    define variable mlgIsGlobal as logical no-undo initial false.
    define variable mlgIsShared as logical no-undo initial false.
    define variable mlgIsNew as logical no-undo initial false.
    define variable mlgIsPrivate as logical no-undo initial false.
    define variable mlgIsProtected as logical no-undo initial false.
    define variable mlgIsPublic as logical no-undo initial false.
    define variable mlgIsVariable as logical no-undo.
    define variable mchQualifier as character no-undo.
    define variable mlgAssignedFrom as logical no-undo initial false.
    define variable mlgAssignedTo as logical no-undo initial false.
    define variable mobChildToken as JPNode no-undo.
    define variable mobSubToken as JPNode no-undo.
    define variable mobSymbol as Symbol no-undo.
    define variable mobToken as JPNode no-undo.
    define variable minWhich as integer no-undo.

    define buffer mbfSharedObject for SharedObject.
    define buffer mbfSharedBrowse for SharedBrowse.
    define buffer mbfSharedBuffer for SharedBuffer.    
    define buffer mbfSharedDataset for SharedDataset.
    define buffer mbfSharedFrame for SharedFrame.
    define buffer mbfSharedMenu for SharedMenu.
    define buffer mbfSharedQuery for SharedQuery.
    define buffer mbfSharedStream for SharedStream.
    define buffer mbfSharedTempTable for SharedTempTable.
    define buffer mbfSharedTempTableField for SharedTempTableField.
    define buffer mbfSharedVariable for SharedVariable.
    define buffer mbfSharedWorkTable for SharedWorkTable.
    define buffer mbfSharedWorkTableField for SharedWorkTableField.                                        
    
    define buffer mbfttField for ttField.
    define buffer mbfDB_DB for DB_DB.
    define buffer mbfDB_Table for DB_Table.
  
    inCurrentLine = ipobNode:getLine().
    mobToken = ipobNode:firstChild().
    mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).
    mchQualifier = mchTokenType.

    case mchTokenType:
      when "NEW"
        then do:
          mlgIsNew = true.
          mchQualifier = "NEW".
          mobChildToken = mobToken:firstChild().
          if NodeTypes:getTypeName(mobChildToken:GetType()) = "GLOBAL"
          then do:
            mlgIsGlobal = true.
            mchQualifier = mchQualifier + " GLOBAL".
            mobChildToken = mobChildToken:nextSibling().
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
            then mlgIsShared = true.
          end. /* if global */
          else do:
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
            then mlgIsShared.
          end. /* not global */
          mchQualifier = mchQualifier + " SHARED".
        end. /* new */
      when "SHARED"
        then do:
          mlgIsShared = true.
        end.         
      when "PRIVATE"
        then do:
          mlgIsPrivate = true.
        end.         
      when "PROTECTED"
        then do:
          mlgIsProtected = true.
        end.         
      when "PUBLIC"
        then do:
          mlgIsPublic = true.
        end.         
    end case.
    
    if not ( mlgIsShared or mlgIsProtected or mlgIsPublic ) then return.  /* Skip non-shared ( for now ) */
    
    mobToken = mobToken:nextSibling().
    mchObjectType = NodeTypes:getTypeName(mobToken:GetType()).
    mobToken = mobToken:nextSibling().
    mchName = mobToken:getText().  /* ID */  
    case mchObjectType:
      when "BROWSE"
        then do:
          mobToken = mobToken:nextSibling().
          mchDescription = TreeUtils:fullPreproText( mobToken ).
        end.    
      when "BUFFER"
        then do:
          mobToken = mobToken:nextSibling().  /* FOR */
          mobToken = mobToken:nextSibling().
          if  NodeTypes:getTypeName(mobToken:GetType())  = "TEMPTABLE"
          then mobToken = mobToken:nextSibling(). 
          mchBufferName = mobToken:getText().
          if mobToken:attrGetS("storetype":U) = "st-dbtable"
          then do:
            mlgIsDBTable = true.
            if num-entries( mchBufferName, "." ) = 2
            then do:
              find mbfDB_DB where mbfDB_DB.chDB_Name = entry( 1, mchBufferName, "." ) no-lock no-error.
              if available( mbfDB_DB ) 
              then do:
                mchDBID = mbfDB_DB.chID.
                find mbfDB_Table 
                    where mbfDB_Table.chDBID = mbfDB_DB.chID
                    and   mbfDB_Table.chTableName = entry( 2, mchBufferName, "." ) 
                    and   mbfDB_Table.chOwner = "PUB":u 
                    no-lock no-error.
                if available( mbfDB_Table ) 
                then do:
                  mchDBID[1] = mbfDB_DB.chID.
                  mchTableID[1] = mbfDB_Table.chID.
                end.
              end.
            end.
            else do:
              find mbfDB_Table where mbfDB_Table.chTableName = mchBufferName no-lock no-error.
              if available( mbfDB_Table ) 
              then do:
                mchDBID = mbfDB_Table.chDBID.
                mchTableID = mbfDB_Table.chID.
              end.               
            end.
          end.
        end.
      when "DATASET"
        then do:
          mobToken = mobToken:nextSibling().  /* FOR */
          mobToken = mobToken:nextSibling().
          mchDescription = TreeUtils:fullPreproText( mobToken ).
        end.
      when "FRAME"
        then do:
          mobToken = mobToken:nextSibling().
          mchDescription = TreeUtils:fullPreproText( mobToken ).
        end.
      when "MENU"
        then do:
          mobToken = mobToken:nextSibling().
          mchDescription = TreeUtils:fullPreproText( mobToken ).  
        end.
      when "PROPERTY"
        then do:
          mlgIsVariable = false.
          mobSymbol = mobToken:getSymbol().
          if mobSymbol:getNumReads() > 0 then mlgAssignedFrom = true.
          if mobSymbol:getNumWrites() > 0 then mlgAssignedTo = true.
          mobSubToken = mobToken:nextSibling().  
          if NodeTypes:getTypeName(mobSubToken:GetType()) = "AS"
          then do:
            mobSubToken = mobSubToken:firstChild().
            mchDataType = NodeTypes:getTypeName(mobSubToken:GetType()).
          end.
          else if NodeTypes:getTypeName(mobSubToken:GetType()) = "LIKE"
          then do:
            mobSubToken = mobSubToken:firstChild().    /* FIELD_REF */
            mobSubToken = mobSubToken:firstChild().    /* ID */
            mchLike = mobSubToken:getText().
          end.
          else do:
            obReport:WriteMessage ( substitute("BuildSharedOne: Confused parsing fields for property &1 at line &2 of &3; got &4 node", mchName, mobSubToken:getLine(), chCurrentCompileUnitName, NodeTypes:getTypeName(mobSubToken:GetType()) )).                      
          end.  
        end.
      when "QUERY"
        then do:
          mobToken = mobToken:nextSibling().  /* FOR */
          mobToken = mobToken:nextSibling().  /* ID */
          mchQueryName = mobToken:getText().  
          mobToken = mobToken:nextSibling().  /* FOR*/  
          minNumRecords = 1.
    
          RECORD_NAME_BLOCK:
          repeat:
            mchRecordName[ minNumRecords ] = mobToken:getText().
            mobBufferSymbol = mobToken:getSymbol().
            mobTableBuffer = cast( mobBufferSymbol, TableBuffer ).
            mobTable = mobTableBuffer:getTable().
            mchTableName[ minNumRecords ] = mobTable:getName().
            minStoreType = mobTable:getStoretype().
            case minStoreType:
              when 1102 then mchStoreType[ minNumRecords ] = "DBTable".
              when 1103 then mchStoreType[ minNumRecords ] = "TempTable".
              when 1104 then mchStoreType[ minNumRecords ] = "WorkTable".
            end case.
            if NodeTypes:getTypeName(mobToken:nextSibling():GetType()) = "FIELDS"
            then do:
              mobToken = mobToken:nextSibling().  /* FIELDS */
              mchFieldPhrase[ minNumRecords ] = RestOfPhrase( mobToken:firstChild() ).
            end.
            
            if mchStoreType[ minNumRecords ] = "DBTable"
            then do:
              mlgIsDBTable[ minNumRecords ] = true.
              if num-entries( mchRecordName[ minNumRecords ], "." ) = 2
              then do:
                find mbfDB_DB where mbfDB_DB.chDB_Name = entry( 1, mchRecordName[ minNumRecords ], "." ) no-lock no-error.
                if available( mbfDB_DB ) 
                then do:
                  mchDBID[ minNumRecords ] = mbfDB_DB.chID.
                  find mbfDB_Table 
                      where mbfDB_Table.chDBID = mchDBID[ minNumRecords ]
                      and   mbfDB_Table.chTableName = mchTableName[ minNumRecords ] 
                      and   mbfDB_Table.chOwner = "PUB":u 
                      no-lock no-error.
                  if available( mbfDB_Table ) then mchTableID[ minNumRecords ] = mbfDB_Table.chID.
                end.
              end.
              else do:
                find mbfDB_Table where mbfDB_Table.chTableName = mchTableID[ minNumRecords ] no-lock no-error.
                if available( mbfDB_Table ) 
                then do:
                  mchDBID[ minNumRecords ] = mbfDB_Table.chDBID.
                  mchTableID[ minNumRecords ] = mbfDB_Table.chID.
                end.               
              end.
            end.            
            
            if NodeTypes:getTypeName(mobToken:nextSibling():GetType()) = "COMMA"
            then do:
              mobToken = mobToken:nextSibling().  /* COMMA */
              mobToken = mobToken:nextSibling().  /* NEXT RECORD_NAME */
              minNumRecords = minNumRecords + 1.
              next RECORD_NAME_BLOCK.
            end.
            else do:
              mobToken = mobToken:nextSibling().
              leave.
            end.
          end.
    
          mchAdditionalInfo = RestOfPhrase( mobToken ).
        end.
      when "STREAM"
        then do:
          /* Nothing to do */          
        end.
      when "TEMPTABLE"
        then do:
          do while NodeTypes:getTypeName(mobToken:GetType()) <> "FIELD": /* Skip No-undo and such */
            mobToken = mobToken:nextSibling().
          end.
          repeat:
            mobSubToken = mobToken:FirstChild().  /* ID */
            create mbfttField.
            assign
                minOrder = minOrder + 1    
                mbfttField.inOrder = minOrder
                mbfttField.chName = mobSubToken:getText()
                .
            mobSubToken = mobSubToken:nextSibling().  
            if NodeTypes:getTypeName(mobSubToken:GetType()) = "AS"
            then do:
              mobSubToken = mobSubToken:firstChild().
              mbfttField.chDataType = NodeTypes:getTypeName(mobSubToken:GetType()).
            end.
            else if NodeTypes:getTypeName(mobSubToken:GetType()) = "LIKE"
            then do:
              mobSubToken = mobSubToken:firstChild().    /* FIELD_REF */
              mobSubToken = mobSubToken:firstChild().    /* ID */
              mbfttField.chLike = mobSubToken:getText().
            end.
            else do:
              obReport:WriteMessage ( substitute("BuildSharedOne: Confused parsing fields for temp-table &1 at line &2 of &3; got &4 node", mchName, mobSubToken:getLine(), chCurrentCompileUnitName, NodeTypes:getTypeName(mobSubToken:GetType()) )).                      
            end.
            mobToken = mobToken:nextSibling().
            if NodeTypes:getTypeName(mobToken:GetType()) <> "FIELD" then leave.
          end.
          /* Indexes are not being captured */
        end.
      when "VARIABLE"
        then do:
          mlgIsVariable = true.
          mobSymbol = mobToken:getSymbol().
          if mobSymbol:getNumReads() > 0 then mlgAssignedFrom = true.
          if mobSymbol:getNumWrites() > 0 then mlgAssignedTo = true.
          mobSubToken = mobToken:nextSibling().  
          if NodeTypes:getTypeName(mobSubToken:GetType()) = "AS"
          then do:
            mobSubToken = mobSubToken:firstChild().
            mchDataType = NodeTypes:getTypeName(mobSubToken:GetType()).
          end.
          else if NodeTypes:getTypeName(mobSubToken:GetType()) = "LIKE"
          then do:
            mobSubToken = mobSubToken:firstChild().    /* FIELD_REF */
            mobSubToken = mobSubToken:firstChild().    /* ID */
            mchLike = mobSubToken:getText().
          end.
          else do:
            obReport:WriteMessage ( substitute("BuildSharedOne: Confused parsing fields for variable &1 at line &2 of &3; got &4 node", mchName, mobSubToken:getLine(), chCurrentCompileUnitName, NodeTypes:getTypeName(mobSubToken:GetType()) )).                      
          end.  
        end.
      when "WORKTABLE"
        then do:
          do while NodeTypes:getTypeName(mobToken:GetType()) <> "FIELD": /* Skip No-undo and such */
            mobToken = mobToken:nextSibling().
          end.
          repeat:
            mobSubToken = mobToken:FirstChild().  /* ID */
            create mbfttField.
            assign
                minOrder = minOrder + 1    
                mbfttField.inOrder = minOrder
                mbfttField.chName = mobSubToken:getText()
                .
            mobSubToken = mobSubToken:nextSibling().  
            if NodeTypes:getTypeName(mobSubToken:GetType()) = "AS"
            then do:
              mobSubToken = mobSubToken:firstChild().
              mbfttField.chDataType = NodeTypes:getTypeName(mobSubToken:GetType()).
            end.
            else if NodeTypes:getTypeName(mobSubToken:GetType()) = "LIKE"
            then do:
              mobSubToken = mobSubToken:firstChild().    /* FIELD_REF */
              mobSubToken = mobSubToken:firstChild().    /* ID */
              mbfttField.chLike = mobSubToken:getText().
            end.
            else do:
              obReport:WriteMessage ( substitute("BuildSharedOne: Confused parsing fields for work-table &1 at line &2 of &3; got &4 node", mchName, mobSubToken:getLine(), chCurrentCompileUnitName, NodeTypes:getTypeName(mobSubToken:GetType()) )).                      
            end.
            mobToken = mobToken:nextSibling().
            if NodeTypes:getTypeName(mobToken:GetType()) <> "FIELD" then leave.
          end.
        end.    
      otherwise
        do:
          obReport:WriteMessage ( substitute("BuildSharedOne: Confused parsing fields at line &1 of &2; got &3 node", mobToken:getLine(), chCurrentCompileUnitName, NodeTypes:getTypeName(mobToken:GetType()) )).                      
        end.
    end case.                                                                                                  

    do for mbfSharedObject:
      find mbfSharedObject no-lock
          where mbfSharedObject.chCompileUnitID = ipchCUID
          and   mbfSharedObject.chName = mchName
          and   mbfSharedObject.chObjectType = mchObjectType
          no-error.
      if not available( mbfSharedObject )
      then do:
        create mbfSharedObject.
        assign
            mbfSharedObject.chID = guid
            mbfSharedObject.chCompileUnitID = ipchCUID
            mbfSharedObject.chName = mchName
            mbfSharedObject.chObjectType = mchObjectType
            mbfSharedObject.lgIsNew = mlgIsNew
            mbfSharedObject.chQualifier = mchQualifier
            mbfSharedObject.chAdditionalInfo = mchAdditionalInfo
            inTotalShared = inTotalShared + 1
            .
      end. 
      else do:
        obReport:WriteMessage ( substitute("BuildSharedOne: Duplicate shared object definition in file &1 for variable &2 of type &3 at line &4", chFullPath, mchName, mchObjectType, ipobNode:getLine() )).        
      end.
      case mchObjectType:
      when "BROWSE"
        then do for mbfSharedBrowse:
          create mbfSharedBrowse.
          assign
              mbfSharedBrowse.chID = guid
              mbfSharedBrowse.chObjectID = mbfSharedObject.chID
              /* chQueryID */
              mbfSharedBrowse.chDescription = mchDescription
              .
        end.    
      when "BUFFER"
        then do for mbfSharedBuffer:
          create mbfSharedBuffer.
          assign
              mbfSharedBuffer.chID = guid
              mbfSharedBuffer.chObjectID = mbfSharedObject.chID
              mbfSharedBuffer.chDBID = mchDBID[ 1 ]
              mbfSharedBuffer.chTableID = mchTableID[ 1 ]
              mbfSharedBuffer.chRecordName = mchRecordName[ 1 ]
              mbfSharedBuffer.lgIsDBTable = mlgIsDBTable[ 1 ]
              .
        end.
      when "DATASET"
        then do for mbfSharedDataset:
          create mbfSharedDataset.
          assign
              mbfSharedDataset.chID = guid
              mbfSharedDataset.chObjectID = mbfSharedObject.chID
              mbfSharedDataset.chDescription = mchDescription
              .
        end.    
      when "FRAME"
        then do for mbfSharedFrame:
          create mbfSharedFrame.
          assign
              mbfSharedFrame.chID = guid
              mbfSharedFrame.chObjectID = mbfSharedObject.chID
              mbfSharedFrame.chDescription = mchDescription
              .
        end.    
      when "MENU"
        then do for mbfSharedMenu:
          create mbfSharedMenu.
          assign
              mbfSharedMenu.chID = guid
              mbfSharedMenu.chObjectID = mbfSharedObject.chID
              mbfSharedMenu.chDescription = mchDescription
              .
        end.    
      when "QUERY"
        then do minWhich = 1 to minNumRecords: 
          do for mbfSharedQuery:
            create mbfSharedQuery.
            assign
                mbfSharedQuery.chID = guid
                mbfSharedQuery.chObjectID = mbfSharedObject.chID
                mbfSharedQuery.chDBID = mchDBID[ minWhich ]
                mbfSharedQuery.chTableID = mchTableID[ minWhich ]
                mbfSharedQuery.chRecordName = mchRecordName[ minWhich ] /* Note Table name is in DB_Table */
                mbfSharedQuery.lgIsDBTable = mlgIsDBTable[ minWhich ]
                mbfSharedQuery.chFieldsPhrase = mchFieldPhrase[ minWhich ]
                .
          end. 
        end.
      when "STREAM"
        then do for mbfSharedStream:
          create mbfSharedStream.
          assign
              mbfSharedStream.chID = guid
              mbfSharedStream.chObjectID = mbfSharedObject.chID
              mbfSharedStream.lgIsGlobal = mlgIsGlobal
              .
        end.    
      when "TEMPTABLE"
        then do for mbfSharedTempTable:
          create mbfSharedTempTable.
          assign
              mbfSharedTempTable.chID = guid
              mbfSharedTempTable.chObjectID = mbfSharedObject.chID
              mbfSharedTempTable.lgIsGlobal = mlgIsGlobal
              .
          for each mbfttField by mbfttField.inOrder:
            do for mbfSharedTempTableField:
              create mbfSharedTempTableField.
              assign
                  mbfSharedTempTableField.chID = guid
                  mbfSharedTempTableField.chSharedTempTableID = mbfSharedTempTable.chID   
                  mbfSharedTempTableField.chName = mbfttField.chName
                  mbfSharedTempTableField.chDataType = mbfttField.chDataType
                  mbfSharedTempTableField.chLike = mbfttField.chLike
                  .       
            end.
          end.
        end.    
      when "VARIABLE"
        then do for mbfSharedVariable:
          create mbfSharedVariable.
          assign
              mbfSharedVariable.chID = guid
              mbfSharedVariable.chObjectID = mbfSharedObject.chID
              mbfSharedVariable.lgIsGlobal = mlgIsGlobal
              mbfSharedVariable.chDataType = mchDataType
              mbfSharedVariable.chLike = mchLike
              mbfSharedVariable.lgAssignedFrom = mlgAssignedFrom
              mbfSharedVariable.lgAssignedTo = mlgAssignedTo
              .
        end.    
      when "WORKTABLE"
        then do for mbfSharedWorkTable:
          create mbfSharedWorkTable.
          assign
              mbfSharedWorkTable.chID = guid
              mbfSharedWorkTable.chObjectID = mbfSharedObject.chID
              .
          for each mbfttField by mbfttField.inOrder:
            do for mbfSharedWorkTableField:
              create mbfSharedWorkTableField.
              assign
                  mbfSharedWorkTableField.chID = guid
                  mbfSharedWorkTableField.chSharedWorkTableID = mbfSharedWorkTable.chID   
                  mbfSharedWorkTableField.chName = mbfttField.chName
                  mbfSharedWorkTableField.chDataType = mbfttField.chDataType
                  mbfSharedWorkTableField.chLike = mbfttField.chLike
                  .       
            end.
          end.
        end.             
      end case.
      empty temp-table ttField.
    end.
    
  end method.

  method private character RestOfPhrase(
      ipobNode as JPNode
      ):
    define variable mchRestOfPhrase as character initial "" no-undo.

    if ipobNode = ? then return mchRestOfPhrase.
    if NodeTypes:getTypeName(ipobNode:getType()) = "PERIOD" then return mchRestOfPhrase.
    if NodeTypes:getTypeName(ipobNode:getType()) = "COMMA" then return mchRestOfPhrase.
    
    mchRestOfPhrase = ipobNode:getText().
    if ipobNode:firstChild() <> ? and NodeTypes:getTypeName(ipobNode:firstChild():getType()) <> "PERIOD"
    then mchRestOfPhrase = mchRestOfPhrase + " " + RestOfPhrase( ipobNode:firstChild() ).
    if ipobNode:nextSibling() <> ? and NodeTypes:getTypeName(ipobNode:nextSibling():getType()) <> "PERIOD"
    then mchRestOfPhrase = mchRestOfPhrase + " " + RestOfPhrase( ipobNode:nextSibling() ).
    return mchRestOfPhrase.
  end method.

end class.