/*------------------------------------------------------------------------
    File        : BuildSubUnits
    Purpose     : Scan code base and build objects for procedures, classes, IPs, methods, functions, includes, etc.
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 15 14:51:48 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildSubUnits inherits BuildDirectoryTree: 
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalIPs as integer no-undo get. private set.
  define public property inTotalMethods as integer no-undo get. private set.
  define public property inTotalFunctions as integer no-undo get. private set.
  define public property inTotalIncludes as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.

  define private variable hnXrefHandler as handle no-undo.
  define private variable chFullPath as character no-undo.
  define private variable chCurrentCompileUnitName as character no-undo.
  define private variable chCurrentCompileSubUnitID as character no-undo.
  define private variable chCurrentCompileSubUnitName as character no-undo.
  define private variable chCurrentIncludeReferenceID as character no-undo.
  define private variable chCurrentIncludeName as character no-undo.
  define private variable chCurrentNodeType as character no-undo.
  define private variable chCurrentModeValue as character no-undo.
  define private variable chCompileUnitID as character no-undo.
  define private variable chLineNo as character no-undo.
  define private variable chCurrentReferenceType as character no-undo.

	constructor public BuildSubUnits ( 
      ipchSourceDirectory as character
      ):  
    super ( ipchSourceDirectory ).
	end constructor.

	destructor public BuildSubUnits ( ):
    if valid-object(obReport) 
    then obReport:WriteMessage ( substitute("End of scan with &1 total files, &2 total IPs, &3 total functions, &4 total methods, and &5 Includes in &6 compiled files",inTotalFiles, inTotalIPs, inTotalFunctions, inTotalMethods, inTotalIncludes, inTotalCompiled ) ).
    log-manager:write-message ( substitute( "End BuildSubUnits with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    super:Initialize().

    log-manager:write-message ("Start BuildSubUnits").
    obReport:WriteMessage ( substitute("Exception list for build subunits of &1", chSourceDirectoryList ) ).

    run com/cintegrity/ABL2DB/SubUnitXrefHandler.p persistent set hnXrefHandler.
    run Initialize in hnXrefHandler ( this-object ).
  end method.
  
  method public void CleanUp():
    if valid-handle(hnXrefHandler)
    then do:
      delete procedure hnXrefHandler.
      hnXrefHandler = ?.
    end.
    else log-manager:write-message ("BuildSubUnits: Destructor: Invalid handle for hnXrefHandler").       
  end method.
  
  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll ().
    
    CleanUp().
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not bfContents.lgIsDir:
      ScanOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.

  method private void ScanOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchXrefFile as character no-undo.
    
    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
    
    inTotalFiles = inTotalFiles + 1.
    
    do for bfDiskFile:
      find bfDiskFile no-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if not available ( bfDiskFile )
      then do:
        obReport:WriteMessage ( substitute("BuildSubUnits: DiskFile not found for &1", ipchFullPath ) ).
        inTotalExceptions = inTotalExceptions + 1.
        return.
      end.
      if bfDiskFile.lgCompilable
      then do for bfCompileUnit:
        find bfCompileUnit no-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
        if available ( bfCompileUnit )
        then do:
          assign
              inTotalCompiled = inTotalCompiled + 1
              mchXrefFile = obPathUtilities:XrefFile( bfCompileUnit.chXrefBaseDir, ipchPackage, ipchName )
              chFullPath = ipchFullPath
              chCurrentCompileUnitName = obPathUtilities:RelativePathFromComponents( ipchPackage, ipchName )
              chCompileUnitID = bfCompileUnit.chID.
              .
            ProcessXref ( mchXrefFile ).
        end. /* available bfCompileUnit */
        else do:
         obReport:WriteMessage  ( substitute("BuildSubLinks: CompileUnit missing for &1", ipchFullPath ) ). 
         inTotalExceptions = inTotalExceptions + 1.
         return.
        end.
      end. /* lgCompilable */
    end. /* for bfDiskFile */
  end method.
  
  method private void ProcessXref ( 
      ipchXrefTarget as character
      ):
    define variable mhnParser as handle no-undo.
  
    file-info:file-name = ipchXrefTarget.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      obReport:WriteMessage ( substitute("BuildSubUnits: XREF file '&1' not found or not readable", ipchXrefTarget ) ).
      inTotalExceptions = inTotalExceptions + 1.
      return.
    end.     
 
    do on error undo, leave:
      create sax-reader mhnParser.
      mhnParser:handler = hnXrefHandler.
      mhnParser:set-input-source("file", ipchXrefTarget ).
      mhnParser:sax-parse() no-error.
      if error-status:error
        then do:
          if error-status:num-messages gt 0
          then log-manager:write-message( error-status:get-message(1)).
          else log-manager:write-message( return-value ).
        end.
      delete object mhnParser.
      mhnParser  = ?.      
      catch eobAppError as Progress.Lang.AppError:
        log-manager:write-message ( eobAppError:ReturnValue ).  
        delete object eobAppError.
        if valid-handle (mhnParser) then delete object mhnParser.
        mhnParser  = ?.
      end catch.
    end.
  end method.
  
  method public void StartElement (
      ipchNameSpaceURI as character,
      ipchLocalName as character,
      ipchElementType as character,
      iphnAttributes as handle
      ):
    define variable mchReferenceType as character no-undo.
    define variable mchIdentifier as character no-undo.
    define variable mchIncludeRelativePath as character no-undo.
    define variable mchIncludePath as character no-undo.
    define variable mchName as character no-undo.
    define variable mchPackage as character no-undo.
    define variable mchSourceDirectory as character no-undo.
      
    define buffer bfCompileSubUnit for CompileSubUnit.
    define buffer bfDiskFile for DiskFile.
    define buffer bfIncludeUnit for IncludeUnit.
    define buffer bfIncludeReference for IncludeReference.
      
    case ipchElementType:
      when "REFERENCE"                           /* Everything is a reference */
        then do:
          chCurrentReferenceType = iphnAttributes:get-value-by-qname("Reference-type").
          mchIdentifier = iphnAttributes:get-value-by-qname("Object-identifier").
          case chCurrentReferenceType:
            when "INCLUDE"                                               /* Include */
              then do:
                mchIncludeRelativePath = trim(replace(mchIdentifier, chr(10), " ")).
                mchIncludeRelativePath = entry( 1, mchIncludeRelativePath, " ").
                mchIncludePath = search( mchIncludeRelativePath ).
                chCurrentIncludeName = mchIncludeRelativePath.
                if mchIncludePath = ?
                then do:
                  obReport:WriteMessage (substitute("BuildSubUnit: Include file &1 not found for &2 (Identifier = &3)", mchIncludeRelativePath, chCurrentCompileUnitName, mchIdentifier)).
                  inTotalExceptions = inTotalExceptions + 1.                
                end.
                else do for bfDiskFile:
                  assign
                      mchSourceDirectory = obPathUtilities:SourceDirectoryFromFullPath( mchIncludePath )
                      mchPackage = obPathUtilities:PackageFromFullPath( mchIncludePath )
                      mchName = obPathUtilities:NameFromPath( mchIncludePath )
                      .
                  find bfDiskFile exclusive-lock
                      where bfDiskFile.chBaseDirectory = mchSourceDirectory
                      and   bfDiskFile.chPackage = mchPackage
                      and   bfDiskFile.chName = mchName
                      no-error.
                  if not available ( bfDiskFile )
                  then do:
                    obReport:WriteMessage (substitute("BuildSubUnit|StartElement: DiskFile not found for &1 in &2", mchIncludePath, chCurrentCompileUnitName)).     
                    inTotalExceptions = inTotalExceptions + 1.                  
                  end.   
                  else do for bfIncludeUnit:
                    bfDiskFile.lgIsInclude = true.
                    find bfIncludeUnit no-lock where bfIncludeUnit.chDiskFileID = bfDiskFile.chID no-error.
                    if not available ( bfIncludeUnit )
                    then do:
                      create bfIncludeUnit.
                      assign
                          bfIncludeUnit.chID =  guid
                          bfIncludeUnit.chDiskFileID = bfDiskFile.chID
                          inTotalIncludes = inTotalIncludes + 1
                          .
                    end.
                    do for bfIncludeReference:
                      create bfIncludeReference.
                      assign
                          bfIncludeReference.chID = guid
                          bfIncludeReference.chCompileUnitID = chCompileUnitID
                          bfIncludeReference.chIncludeUnitID = bfIncludeUnit.chID
                          chCurrentIncludeReferenceID = bfIncludeReference.chID
                          .
                    end. /* of bfIncludeReference */
                  end. /* of bfIncludeUnit */
                end. /* of bfDiskFile */
              end. /* of when */
            when "FUNCTION"                                             /* Function */
              then do for bfCompileSubUnit:
                create bfCompileSubUnit.
                assign
                    bfCompileSubUnit.chID = guid
                    bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                    bfCompileSubUnit.chName = mchIdentifier
                    bfCompileSubUnit.chType = "Function"
                    bfCompileSubUnit.lgIsPrivate = no
                    bfCompileSubUnit.lgIsProtected = no
                    chCurrentCompileSubUnitName = mchIdentifier
                    chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                    inTotalFunctions = inTotalFunctions + 1
                    .
              end.
            when "PRIVATE-FUNCTION"                             /* Private Function */
              then do for bfCompileSubUnit:
                create bfCompileSubUnit.
                assign
                    bfCompileSubUnit.chID = guid
                    bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                    bfCompileSubUnit.chName = mchIdentifier
                    bfCompileSubUnit.chType = "Function"
                    bfCompileSubUnit.lgIsPrivate = yes
                    bfCompileSubUnit.lgIsProtected = no
                    chCurrentCompileSubUnitName = mchIdentifier
                    chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                    inTotalFunctions = inTotalFunctions + 1
                    .
              end.
            when "PROCEDURE"                                           /* Procedure */
              then do for bfCompileSubUnit:
                create bfCompileSubUnit.
                assign
                    bfCompileSubUnit.chID = guid
                    bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                    bfCompileSubUnit.chName = mchIdentifier
                    bfCompileSubUnit.chType = "IP"
                    bfCompileSubUnit.lgIsPrivate = no
                    bfCompileSubUnit.lgIsProtected = no
                    chCurrentCompileSubUnitName = mchIdentifier
                    chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                    inTotalIPs = inTotalIPs + 1
                    .
              end.
            when "PRIVATE-PROCEDURE"                           /* Private Procedure */
              then do for bfCompileSubUnit:
                create bfCompileSubUnit.
                assign
                    bfCompileSubUnit.chID = guid
                    bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                    bfCompileSubUnit.chName = mchIdentifier
                    bfCompileSubUnit.chType = "IP"
                    bfCompileSubUnit.lgIsPrivate = yes
                    bfCompileSubUnit.lgIsProtected = no
                    chCurrentCompileSubUnitName = mchIdentifier
                    chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                    inTotalIPs = inTotalIPs + 1
                    .
              end.   
            when "METHOD"                                                 /* Method */
              then do for bfCompileSubUnit:
                create bfCompileSubUnit.
                assign
                    bfCompileSubUnit.chID = guid
                    bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                    bfCompileSubUnit.chName = mchIdentifier
                    bfCompileSubUnit.chType = "Method"
                    bfCompileSubUnit.lgIsPrivate = no
                    bfCompileSubUnit.lgIsProtected = no
                    chCurrentCompileSubUnitName = mchIdentifier
                    chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                    inTotalMethods = inTotalMethods + 1
                    .             
              end.      
            otherwise do:
              /* Nothing since other reference types are handled elsewhere */
            end.            
          end case. /* Reference type */
        end. /* REFERENCE */
      when "Access-mode"
        then do:
          chCurrentNodeType = "Access-mode".
          chCurrentModeValue = "".
        end.
      when "Line-num"
        then do:
          chCurrentNodeType = "Line-num".
          chLineNo = "".
        end.
      otherwise do:
        chCurrentNodeType = "".
      end.
    end. /* case ipchElementType */        
  end method.
  
  method public void Characters (
      ipchData as memptr,
      ipinNumChars as integer
      ):
    case chCurrentNodeType:
      when "Access-mode" then chCurrentModeValue = chCurrentModeValue + GET-STRING(ipchData, 1, get-size(ipchData)). 
      when "Line-num" then chLineNo = chLineNo + GET-STRING(ipchData, 1, get-size(ipchData)). 
    end case.
  end method.      

  method public void EndElement (
      ipchNameSpaceURI as character,
      ipchLocalName as character,  
      ipchElementType as character
      ):
    define buffer bfCompileSubUnit for CompileSubUnit.

    case ipchElementType:
      when "REFERENCE"                           /* Everything is a reference */
        then do:    
          case chCurrentReferenceType:
            when "INCLUDE"
              then do:
                UpdateIncludeLineNo().
              end.
            when "FUNCTION"                                             /* Function */
              then do:
                UpdateSubUnitLineNo().
              end.
            when "PRIVATE-FUNCTION"                             /* Private Function */
              then do:
                UpdateSubUnitLineNo().
              end.
            when "PROCEDURE"                                           /* Procedure */
              then do:
                UpdateSubUnitLineNo().
              end.
            when "PRIVATE-PROCEDURE"                           /* Private Procedure */
              then do:
                UpdateSubUnitLineNo().
              end.      
            when "METHOD"                                      /* Private Procedure */
              then do:
                UpdateSubUnitLineNo().
              end.        
            otherwise do:
              /* Nothing since other reference types are handled elsewhere */
            end.
          end case.
        end. /* REFERENCE */
      when "Access-mode"
        then do :
          if chCurrentCompileSubUnitName > ""
          then do for bfCompileSubUnit:
            find bfCompileSubUnit exclusive-lock where bfCompileSubUnit.chID = chCurrentCompileSubUnitID no-error.
            if not available( bfCompileSubUnit )
            then do:
              obReport:WriteMessage (substitute("BuildSubUnit: CompileSubUnit not found in Access-Mode for &1 of &2", chFullPath, chCurrentCompileSubUnitName)).
              inTotalExceptions = inTotalExceptions + 1.
            end.   
            else do:
              case chCurrentModeValue:
                when ""
                  then do:
                    bfCompileSubUnit.lgIsPrivate = no.
                    bfCompileSubUnit.lgIsProtected = no.
                  end.
                when "PUBLIC"
                  then do:
                    bfCompileSubUnit.lgIsPrivate = no.
                    bfCompileSubUnit.lgIsProtected = no.
                  end.
                when "PRIVATE"
                  then do:
                    bfCompileSubUnit.lgIsPrivate = yes.
                    bfCompileSubUnit.lgIsProtected = no.
                  end.
                when "PROTECTED"
                  then do:
                    bfCompileSubUnit.lgIsPrivate = no.
                    bfCompileSubUnit.lgIsProtected = yes.
                  end.
              end case.      
            end. /* else */
          end. /* CCSU > "" */
        end. /* when "Access-mode" */
      otherwise do:
        /* Nothing since other reference types are handled elsewhere */
      end.
    end.
  end method.      

  method private void UpdateIncludeLineNo ( ):
    define variable inLineNo as integer no-undo.
    define buffer bfIncludeReference for IncludeReference.
    
    find bfIncludeReference exclusive-lock where bfIncludeReference.chID = chCurrentIncludeReferenceID no-error.
    if not available bfIncludeReference
    then do:
      obReport:WriteMessage (substitute("BuildSubUnit: IncludeReference not found in UpdateIncludeLineNo for &1 of &2", chFullPath, chCurrentIncludeName)).
      inTotalExceptions = inTotalExceptions + 1.
    end.   
    else do:
      assign inLineNo = integer(trim(chLineNo)) no-error.
      if error-status:error
      then do:
       obReport:WriteMessage (substitute("BuildSubUnit: Error converting line no = '&1' in UpdateIncludeLineNo for &2 of &3", chLineNo, chFullPath, chCurrentIncludeName)).
      inTotalExceptions = inTotalExceptions + 1.
      end.
      else bfIncludeReference.inLineNo = inLineNo.      
    end. 
  end method.    
  
  method private void UpdateSubUnitLineNo ( ):
    define variable inLineNo as integer no-undo.
    define buffer bfCompileSubUnit for CompileSubUnit.
    
    find bfCompileSubUnit exclusive-lock where bfCompileSubUnit.chID = chCurrentCompileSubUnitID no-error.
    if not available bfCompileSubUnit
    then do:
      obReport:WriteMessage (substitute("BuildSubUnit: CompileSubUnit not found in UpdateSubUnitLineNo for &1 of &2", chFullPath, chCurrentCompileSubUnitName)).
      inTotalExceptions = inTotalExceptions + 1.
    end.   
    else do:
      assign inLineNo = integer(trim(chLineNo)) no-error.
      if error-status:error
      then do:
        obReport:WriteMessage (substitute("BuildSubUnit: Error converting line no = '&1' in UpdateSubUnitLineNo for &2 of &3", chLineNo, chFullPath, chCurrentCompileSubUnitName)).
        inTotalExceptions = inTotalExceptions + 1.
      end.
      else bfCompileSubUnit.inLineNo = inLineNo.      
    end. 
  end method.     

end class.