 /*------------------------------------------------------------------------
    File        : BuildTableLinks
    Purpose     : Scan code base and build links for tables and fields
    Syntax      : 
    Description : 
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Apr 15 14:51:48 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.
using com.cintegrity.ABL2DB.SchemaUtilities.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.BuildTableLinks inherits BuildDirectoryTree: 
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalCompiled as integer no-undo get. private set.
  define public property inTotalTableLinks as integer no-undo get. private set.
  define public property inTotalColumnLinks as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.
  
  define private variable obSchemaUtilities as class SchemaUtilities no-undo.
  define private variable hnXrefHandler as handle no-undo.
  define private variable chCompileUnitID as character no-undo.
  define private variable chDiskFileID as character no-undo.
  define private variable chCompileUnitName as character no-undo.
  define private variable chSourceUnitFullPath as character no-undo.
  define private variable chCurrentReferenceType as character no-undo.
  define private variable chCurrentAccessObject as character no-undo.
  define private variable chDataMemberRef as character no-undo.
  define private variable chTempRef as character no-undo.
  
  define private variable chCurrentNodeType as character no-undo.
  define private variable chCurrentModeValue as character no-undo.
  define private variable chContext as character no-undo.
  define private variable chCurrentProgramCallID as character no-undo.
  define private variable chLineNo as character no-undo.
  define private variable chFullPath as character no-undo.
  define private variable obJavaFile as class java.io.File.
  define private variable obParseUnit as class ParseUnit.
  
	constructor public BuildTableLinks ( 
      ipchSourceDirectory as character
      ):  
    super ( ipchSourceDirectory ).
	end constructor.

	destructor public BuildTableLinks ( ):
	  if valid-object(obSchemaUtilities) then delete object obSchemaUtilities.
    obReport:WriteMessage ( substitute("End of scan with &1 total files; &2 compiled with &3 table links and &4 column links",inTotalFiles, inTotalCompiled, inTotalTableLinks, inTotalColumnLinks ) ).
    log-manager:write-message ( substitute( "End of BuildTableLinks with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    super:Initialize().
 
    log-manager:write-message ("Start BuildTableLinks").
    obReport:WriteMessage ( substitute("Exception list for link build of &1", chSourceDirectoryList ) ).

    run com/cintegrity/ABL2DB/TableLinkXrefHandler.p persistent set hnXrefHandler.
    run Initialize in hnXrefHandler ( this-object ).
    obSchemaUtilities = new SchemaUtilities ().
  end method.
  
  method public void CleanUp():
    if valid-handle(hnXrefHandler)
    then do:
      delete procedure hnXrefHandler.
      hnXrefHandler = ?.
    end.
    else log-manager:write-message ("BuildTableLinks: Destructor: Invalid handle for hnXrefHandler").
  end.
  
  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll ().
    
    CleanUp().
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not bfContents.lgIsDir:
      ScanOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.

  method private void ScanOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchXrefFile as character no-undo.
    
    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
    
    inTotalFiles = inTotalFiles + 1.
    
    do for bfDiskFile:
      find bfDiskFile no-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if not available ( bfDiskFile )
      then do:
        obReport:WriteMessage ( substitute("BuildTableLinks: DiskFile missing for &1", ipchFullPath ) ). 
        inTotalExceptions = inTotalExceptions + 1.
        return.
      end.
      if bfDiskFile.lgCompilable
      then do for bfCompileUnit:
        find bfCompileUnit no-lock where bfCompileUnit.chDiskFileID = bfDiskFile.chID no-error.
        if available ( bfCompileUnit )
        then do:
          assign
              inTotalCompiled = inTotalCompiled + 1
              mchXrefFile = obPathUtilities:XrefFile( bfCompileUnit.chXrefBaseDir, ipchPackage, ipchName )
              chCompileUnitName = obPathUtilities:RelativePathFromComponents( ipchPackage, ipchName )
              chCompileUnitID = bfCompileUnit.chID
              chFullPath = ipchFullPath
              .
          ProcessXref ( mchXrefFile ).
        end. /* available bfCompileUnit */
        else do:
         obReport:WriteMessage  ( substitute("BuildTableLinks: CompileUnit missing for &1", ipchFullPath ) ). 
         inTotalExceptions = inTotalExceptions + 1.
         return.
        end.
      end. /* lgCompilable */
    end. /* for bfDiskFile */
  end method.
  
  method private void ProcessXref ( 
      ipchXrefTarget as character
      ):
    define variable mhnParser as handle no-undo.
  
    file-info:file-name = ipchXrefTarget.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      obReport:WriteMessage ( substitute("BuildTableLinks: XREF file '&1' not found or not readable", ipchXrefTarget ) ).
      inTotalExceptions = inTotalExceptions + 1.
      return.
    end.     
 
    do on error undo, leave:
      create sax-reader mhnParser.
      mhnParser:handler = hnXrefHandler.
      mhnParser:set-input-source("file", ipchXrefTarget ).
      mhnParser:sax-parse() no-error.
      if error-status:error
        then do:
          if error-status:num-messages gt 0
          then log-manager:write-message( error-status:get-message(1)).
          else log-manager:write-message( return-value ).
        end.
      delete object mhnParser.
      mhnParser  = ?.      
      catch eobAppError as Progress.Lang.AppError:
        log-manager:write-message ( eobAppError:ReturnValue ).  
        delete object eobAppError.
        if valid-handle (mhnParser) then delete object mhnParser.
        mhnParser  = ?.
      end catch.
    end.
  end method.
  
  method public void StartElement (
      ipchNameSpaceURI as character,
      ipchLocalName as character,
      ipchElementType as character,
      iphnAttributes as handle
      ):
    define variable lchIdentifier as character no-undo.
    define variable mchDatabaseName as character no-undo.
    define variable mchTableName as character no-undo.
    define variable minWhich as integer no-undo.

    define buffer bfDB_DB for DB_DB.
    define buffer bfDB_Table for DB_Table.
    define buffer bfCU_TableRef for CU_TableRef.
      
    case ipchElementType:
      when "REFERENCE"                          /* All major nodes are a reference */
        then do:
          chCurrentReferenceType = iphnAttributes:get-value-by-qname("Reference-type").
          lchIdentifier = trim(iphnAttributes:get-value-by-qname("Object-identifier")).
          case chCurrentReferenceType:
            when "ACCESS"                                               /* DB Access*/ 
              then do:
                chCurrentAccessObject = lchIdentifier.
              end.
            when "CREATE"                                        /* record creation */ 
              then do:
                chCurrentAccessObject = lchIdentifier.
              end.
            when "DELETE"                                        /* record deletion */ 
              then do:
                chCurrentAccessObject = lchIdentifier.                
              end.
            when "UPDATE"                           /* record or field modification */ 
              then do:
                chCurrentAccessObject = lchIdentifier.                
              end.
          end case. /* Reference Type */
        end. /* REFERENCE */
                                                   /* Properties within a reference */
      when "Data-member-ref"
        then do:
          chCurrentNodeType = "Data-member-ref".
          chDataMemberRef = "".
        end.
      when "Line-num"
        then do:
          chCurrentNodeType = "Line-num".
          chLineNo = "".
        end.
      when "Object-context"
        then do:
          chCurrentNodeType = "Object-context".
          chContext = "".
        end.
      when "Temp-ref"
        then do:
          chCurrentNodeType = "Temp-ref".
          chTempRef = "".
        end.
      otherwise do:
        chCurrentNodeType = "".
      end.
    end. /* case ipchElementType */        
  end method.

  method public void Characters(
      ipchData as memptr,
      ipinNumChars as integer
      ):
    case chCurrentNodeType:
      when "Data-member-ref" then chDataMemberRef = chDataMemberRef + GET-STRING(ipchData, 1, get-size(ipchData)).
           /* D = Class data member
              I = Inherited data member
              P = Public data member.     */ 
      when "Line-num" then chLineNo = chLineNo + GET-STRING(ipchData, 1, get-size(ipchData)).
      when "Object-context" then chContext = chContext + GET-STRING(ipchData, 1, get-size(ipchData)).
      when "Type-ref" then chTempRef = chTempRef + GET-STRING(ipchData, 1, get-size(ipchData)).
           /* T = Temp-table
              W = Workfile   */ 
    end case.  
  end method.

  method public void EndElement(
      ipchNameSpaceURI as character,
      ipchLocalName as character,
      ipchElementType as character
      ):
    define variable mchDatabaseName as character no-undo.
    define variable mchTableName as character no-undo.
    define variable minLineNo as integer no-undo.
    define variable minWhich as integer no-undo.
  
    define buffer bfDB_DB for DB_DB.
    define buffer bfDB_Table for DB_Table.
    define buffer bfDB_Column for DB_Column.
    define buffer bfCU_TableRef for CU_TableRef.
    define buffer bfCU_ColumnRef for CU_ColumnRef.
    
    assign minLineNo = integer(trim(chLineNo)) no-error.
    
    case ipchElementType:
      when "REFERENCE"                           /* All major nodes are a reference */
        then do:
          case chCurrentReferenceType:
            when "ACCESS"                                               /* DB Access*/ 
              then do on error undo, throw:
                if num-entries(chContext,".") > 1         /* Is it a database field */
                then do for bfDB_DB:
                  mchDatabaseName = trim(entry(1, chContext, ".")).
                  mchTableName = trim(entry(2, chContext, ".")).
                  if substring(mchTableName, 1, 1) = "_" 
                  then mchDatabaseName = obSchemaUtilities:MassageMetaDBName ( mchDatabaseName ).
                  find bfDB_DB no-lock where bfDB_DB.chDB_Name = mchDatabaseName no-error.
                  if not available( bfDB_DB )
                  then do:
                    obReport:WriteMessage ( substitute("BuildTableLinks ERROR: DB_DB '&1' not found in &2 from line &3", mchDatabaseName, chCompileUnitName, minLineNo )).
                    inTotalExceptions = inTotalExceptions + 1.
                  end.            
                  else do for bfDB_Table:
                    find bfDB_Table no-lock 
                        where bfDB_Table.chDBID = bfDB_DB.chID 
                        and bfDB_Table.chTableName = mchTableName 
                        no-error.
                    if not available( bfDB_Table )
                    then do:
                      obReport:WriteMessage ( substitute("BuildTableLinks ACCESS ERROR: DB_Table '&1' of DB &2 not found in &3 from line &4", mchTableName, mchDatabaseName, chCompileUnitName, minLineNo )).
                      inTotalExceptions = inTotalExceptions + 1.
                    end.
                    else do for bfCU_TableRef:
                      find bfCU_TableRef no-lock
                          where bfCU_TableRef.chTableID = bfDB_Table.chID
                          and   bfCU_TableRef.chCompileUnitID = chCompileUnitID
                          no-error.
                      if not available( bfCU_TableRef )
                      then do:
                        create bfCU_TableRef.
                        assign 
                            bfCU_TableRef.chID = guid
                            bfCU_TableRef.chTableID = bfDB_Table.chID
                            bfCU_TableRef.chCompileUnitID = chCompileUnitID
                            inTotalTableLinks = inTotalTableLinks + 1
                            .
                      end. /* not available bfCU_TableRef */
                      if chCurrentAccessObject = "" then chCurrentAccessObject = "RECID".
                      do for bfDB_Column:
                        find bfDB_Column no-lock
                            where bfDB_Column.chTableID = bfDB_Table.chID
                            and   bfDB_Column.chColumnName = chCurrentAccessObject
                            no-error.
                        if not available ( bfDB_Column )
                        then do:
                          obReport:WriteMessage ( substitute("BuildTableLinks ERROR: Column '&1.&2' not found in &3 from line &4", mchTableName, chCurrentAccessObject, chCompileUnitName, minLineNo )).
                          inTotalExceptions = inTotalExceptions + 1.
                        end.
                        else do for bfCU_ColumnRef:
                          find bfCU_ColumnRef no-lock
                              where bfCU_ColumnRef.chColumnID = bfDB_Column.chID
                              and   bfCU_ColumnRef.chCompileUnitID = chCompileUnitID
                              no-error.
                          if not available( bfCU_ColumnRef )
                          then do:
                            create bfCU_ColumnRef.
                            assign 
                                bfCU_ColumnRef.chID = guid
                                bfCU_ColumnRef.chColumnID = bfDB_Column.chID
                                bfCU_ColumnRef.chCompileUnitID = chCompileUnitID
                                inTotalColumnLinks = inTotalColumnLinks + 1
                                .
                          end. /* available bfCU_ColumnRef */
                        end. /* create column link */
                      end. /* DB_Column */
                    end. /* for CU_TableRef */
                  end. /* for DB_Table */
                end. /* database field */
                catch eobSysError as Progress.Lang.SysError :
                  log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: ACCESS: DB = &1, Table = &2, (&3), Error = &4", mchDatabaseName, mchTableName, chContext, eobSysError:getMessage(1) ), "SYSERROR").
                  if eobSysError:NumMessages > 1
                  then do minWhich = 2 to eobSysError:NumMessages:
                    log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: ACCESS (cont.): &1", eobSysError:getMessage(minWhich)), "SYSERROR").
                  end.
                  delete object eobSysError no-error.
                  undo, throw new Progress.Lang.AppError( "BuildTableLinks: Aborting." ).             
                end catch.            
              end. /* ACCESS */
            when "CREATE"                                        /* record creation */ 
              then do on error undo, throw:
                if chDataMemberRef = ""
                then do:
                  if num-entries(chCurrentAccessObject,".") > 1         /* Is it a database field */
                  then do for bfDB_DB:
                    mchDatabaseName = trim(entry(1, chCurrentAccessObject, ".")).
                    mchTableName = trim(entry(2, chCurrentAccessObject, ".")).
                    if substring(mchTableName, 1, 1) = "_" 
                    then mchDatabaseName = obSchemaUtilities:MassageMetaDBName ( mchDatabaseName ).
                    find bfDB_DB no-lock where bfDB_DB.chDB_Name = mchDatabaseName no-error.
                    if not available( bfDB_DB )
                    then do:
                      obReport:WriteMessage ( substitute("BuildTableLinks ERROR: DB_DB '&1' not found in CREATE for &2 (&3) at line &4", mchDatabaseName, chCompileUnitName, chCurrentAccessObject, minLineNo )).
                      inTotalExceptions = inTotalExceptions + 1.
                    end.            
                    else do for bfDB_Table:
                      find bfDB_Table no-lock 
                          where bfDB_Table.chDBID = bfDB_DB.chID 
                          and bfDB_Table.chTableName = mchTableName 
                          no-error.
                      if not available( bfDB_Table )
                      then do:
                        obReport:WriteMessage ( substitute("BuildTableLinks CREATE ERROR: DB_Table '&1' of DB &2 not found in &3 at line &4", mchTableName, mchDatabaseName, chCompileUnitName, minLineNo )).
                        inTotalExceptions = inTotalExceptions + 1.
                      end.
                      else do for bfCU_TableRef:
                        find bfCU_TableRef exclusive-lock
                            where bfCU_TableRef.chTableID = bfDB_Table.chID
                            and   bfCU_TableRef.chCompileUnitID = chCompileUnitID
                            no-error.
                        if not available( bfCU_TableRef )
                        then do:
                          create bfCU_TableRef.
                          assign 
                              bfCU_TableRef.chID = guid
                              bfCU_TableRef.chTableID = bfDB_Table.chID
                              bfCU_TableRef.chCompileUnitID = chCompileUnitID
                              inTotalTableLinks = inTotalTableLinks + 1
                              .
                        end. /* not available bfCU_TableRef */ 
                        bfCU_TableRef.lgCreate = true.
                      end. /* for CU_TableRef */
                    end. /* for DB_Table */
                  end. /* is DB field */   
                end.  /* DataMemberRef = "" */
                catch eobSysError as Progress.Lang.SysError :
                  log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: CREATE: DB = &1, Table = &2, (&3), Error = &4", mchDatabaseName, mchTableName, chCurrentAccessObject, eobSysError:getMessage(1) ), "SYSERROR").
                  if eobSysError:NumMessages > 1
                  then do minWhich = 2 to eobSysError:NumMessages:
                    log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: CREATE (cont.): &1", eobSysError:getMessage(minWhich)), "SYSERROR").
                  end.
                  delete object eobSysError no-error.
                  undo, throw new Progress.Lang.AppError( "BuildTableLinks: Aborting." ).             
                end catch.        
              end.
            when "DELETE"                                        /* record deletion */ 
              then do on error undo, throw:
                if chDataMemberRef = ""
                then do:
                  if num-entries(chCurrentAccessObject,".") > 1         /* Is it a database field */
                  then do for bfDB_DB:
                    mchDatabaseName = trim(entry(1, chCurrentAccessObject, ".")).
                    mchTableName = trim(entry(2, chCurrentAccessObject, ".")).
                    if substring(mchTableName, 1, 1) = "_" 
                    then mchDatabaseName = obSchemaUtilities:MassageMetaDBName ( mchDatabaseName ).
                    find bfDB_DB no-lock where bfDB_DB.chDB_Name = mchDatabaseName no-error.
                    if not available( bfDB_DB )
                    then do:
                      obReport:WriteMessage ( substitute("BuildTableLinks ERROR: DB_DB '&1' not found for DELETE in &2 (&3) at line &4", mchDatabaseName, chCompileUnitName, chCurrentAccessObject, minLineNo )).
                      inTotalExceptions = inTotalExceptions + 1.
                    end.            
                    else do for bfDB_Table:
                      find bfDB_Table no-lock 
                          where bfDB_Table.chDBID = bfDB_DB.chID 
                          and bfDB_Table.chTableName = mchTableName 
                          no-error.
                      if not available( bfDB_Table )
                      then do:
                        obReport:WriteMessage ( substitute("BuildTableLinks DELETE ERROR: DB_Table '&1' of DB &2 not found in &3 at line &4", mchTableName, mchDatabaseName, chCompileUnitName, minLineNo )).
                        inTotalExceptions = inTotalExceptions + 1.
                      end.
                      else do for bfCU_TableRef:
                        find bfCU_TableRef exclusive-lock
                            where bfCU_TableRef.chTableID = bfDB_Table.chID
                            and   bfCU_TableRef.chCompileUnitID = chCompileUnitID
                            no-error.
                        if not available( bfCU_TableRef )
                        then do:
                          create bfCU_TableRef.
                          assign 
                              bfCU_TableRef.chID = guid
                              bfCU_TableRef.chTableID = bfDB_Table.chID
                              bfCU_TableRef.chCompileUnitID = chCompileUnitID
                              inTotalTableLinks = inTotalTableLinks + 1
                              .
                        end. /* not available bfCU_TableRef */ 
                        bfCU_TableRef.lgDelete = true.
                      end. /* for CU_TableRef */
                    end. /* for DB_Table */
                  end. /* is DB field */    
                end. /* DataMemberRef = "" */
                catch eobSysError as Progress.Lang.SysError :
                  log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: DELETE: DB = &1, Table = &2, (&3), Error = &4", mchDatabaseName, mchTableName, chCurrentAccessObject, eobSysError:getMessage(1) ), "SYSERROR").
                  if eobSysError:NumMessages > 1
                  then do minWhich = 2 to eobSysError:NumMessages:
                    log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: DELETE (cont.): &1", eobSysError:getMessage(minWhich)), "SYSERROR").
                  end.
                  delete object eobSysError no-error.
                  undo, throw new Progress.Lang.AppError( "BuildTableLinks: Aborting." ).             
                end catch.            
              end.
            when "UPDATE"                           /* record or field modification */ 
              then do on error undo, throw:
                if num-entries(chContext,".") > 1         /* Is it a database field */
                then do for bfDB_DB:
                  mchDatabaseName = trim(entry(1, chContext, ".")).
                  mchTableName = trim(entry(2, chContext, ".")).
                  if substring(mchTableName, 1, 1) = "_" 
                  then mchDatabaseName = obSchemaUtilities:MassageMetaDBName ( mchDatabaseName ).
                  find bfDB_DB no-lock where bfDB_DB.chDB_Name = mchDatabaseName no-error.
                  if not available( bfDB_DB )
                  then do:
                    obReport:WriteMessage ( substitute("BuildTableLinks ERROR: DB_DB '&1' not found in &2 from line &3", mchDatabaseName, chCompileUnitName, minLineNo )).
                    inTotalExceptions = inTotalExceptions + 1.
                  end.            
                  else do for bfDB_Table:
                    find bfDB_Table no-lock 
                        where bfDB_Table.chDBID = bfDB_DB.chID 
                        and bfDB_Table.chTableName = mchTableName 
                        no-error.
                    if not available( bfDB_Table )
                    then do:
                      obReport:WriteMessage ( substitute("BuildTableLinks UPDATE ERROR: DB_Table '&1' of DB &2 not found in &3 from line &4", mchTableName, mchDatabaseName, chCompileUnitName, minLineNo )).
                      inTotalExceptions = inTotalExceptions + 1.
                    end.
                    else do for bfCU_TableRef:
                      find bfCU_TableRef no-lock
                          where bfCU_TableRef.chTableID = bfDB_Table.chID
                          and   bfCU_TableRef.chCompileUnitID = chCompileUnitID
                          no-error.
                      if not available( bfCU_TableRef )
                      then do:
                        create bfCU_TableRef.
                        assign 
                            bfCU_TableRef.chID = guid
                            bfCU_TableRef.chTableID = bfDB_Table.chID
                            bfCU_TableRef.chCompileUnitID = chCompileUnitID
                            inTotalTableLinks = inTotalTableLinks + 1
                            .
                      end. /* not available bfCU_TableRef */
                      if chCurrentAccessObject = "" then chCurrentAccessObject = "RECID".
                      do for bfDB_Column:
                        find bfDB_Column no-lock
                            where bfDB_Column.chTableID = bfDB_Table.chID
                            and   bfDB_Column.chColumnName = chCurrentAccessObject
                            no-error.
                        if not available ( bfDB_Column )
                        then do:
                          obReport:WriteMessage ( substitute("BuildTableLinks ERROR: Column '&1.&2' not found in &3 from line &4", mchTableName, chCurrentAccessObject, chCompileUnitName, minLineNo )).
                          inTotalExceptions = inTotalExceptions + 1.
                        end.
                        else do for bfCU_ColumnRef:
                          find bfCU_ColumnRef exclusive-lock
                              where bfCU_ColumnRef.chColumnID = bfDB_Column.chID
                              and   bfCU_ColumnRef.chCompileUnitID = chCompileUnitID
                              no-error.
                          if not available( bfCU_ColumnRef )
                          then do:
                            create bfCU_ColumnRef.
                            assign 
                                bfCU_ColumnRef.chID = guid
                                bfCU_ColumnRef.chColumnID = bfDB_Column.chID
                                bfCU_ColumnRef.chCompileUnitID = chCompileUnitID
                                inTotalColumnLinks = inTotalColumnLinks + 1
                                .
                          end. /* available bfCU_ColumnRef */
                          bfCU_ColumnRef.lgUpdate = true.
                        end. /* create column link */
                      end. /* DB_Column */
                    end. /* for CU_TableRef */
                  end. /* for DB_Table */
                end. /* database field */
                catch eobSysError as Progress.Lang.SysError :
                  log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: UPDATE: DB = &1, Table = &2, (&3), Error = &4", mchDatabaseName, mchTableName, chContext, eobSysError:getMessage(1) ), "SYSERROR").
                  if eobSysError:NumMessages > 1
                  then do minWhich = 2 to eobSysError:NumMessages:
                    log-manager:write-message ( substitute("BuildTableLinks: EXCEPTION: UPDATE (cont.): &1", eobSysError:getMessage(minWhich)), "SYSERROR").
                  end.
                  delete object eobSysError no-error.
                  undo, throw new Progress.Lang.AppError( "BuildTableLinks: Aborting." ).             
                end catch.            
              end. /* UPDATE */
          end case. /* CurrentReferenceType */
        end. /* REFERENCE */    
      otherwise do:
        /* Nothing */
      end.
    end case.  /* Element type */    
    chCurrentNodeType = "".    
  end method.
  
end class.