 /*------------------------------------------------------------------------
    File        : CompileDirectoryTree
    Purpose     : 
    Syntax      : 
    Description : Compile all programs under a given directory
    Author(s)   : Thomas Mercer-Hursh
    Created     : Thu Apr 10 16:29:20 CDT 2014
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

block-level on error undo, throw.

class com.cintegrity.ABL2DB.CompileDirectoryTree inherits BuildDirectoryTree: 
  define public property chRunDirectoryList as character no-undo get. set.
  define public property chXrefDirectoryList as character no-undo get. set.
  define public property chListDirectoryList as character no-undo get. set.
  define public property chDebugDirectoryList as character no-undo get. set. 
  define public property chPropath as character no-undo get. set.
  
  define public property lgReportErrors as logical no-undo initial true get. set.
  define public property lgReportWarnings as logical no-undo initial true get. set.
  define public property inTotalCompiles as integer no-undo get. private set.
  define public property inTotalErrors as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.
		
	constructor public CompileDirectoryTree ( 
	    ipchSourceDirectoryList as character
      ):	
    super ( ipchSourceDirectoryList ).
		
	end constructor.
	
	destructor public CompileDirectoryTree ():
    if valid-object(obReport) 
    then obReport:WriteMessage (substitute("Total Compiles attempted = &1 of which &2 had errors", inTotalCompiles, inTotalErrors )).
    log-manager:write-message ( substitute ("End CompileDirectory Tree with &1 exceptions", inTotalExceptions ) ).
	end.

  method public override void Initialize():
    define variable minWhich as integer no-undo.
    
    super:Initialize().
  
    log-manager:write-message ("Start CompileDirectory Tree").
    
    do minWhich = 1 to num-entries( chSourceDirectoryList ):
    
      file-info:file-name = entry( minWhich, chRunDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("CompileDirectoryTree ERROR: Run Path '&1' not found or not writeable", entry( minWhich, chRunDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "CompileDirectoryTree: Unable to initialize." ).  
      end.      
      
      file-info:file-name = entry( minWhich, chXrefDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("CompileDirectoryTree ERROR: XREF Path '&1' not found or not writeable", entry( minWhich, chXrefDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "CompileDirectoryTree: Unable to initialize." ).  
      end.     
      
      file-info:file-name = entry( minWhich, chListDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("CompileDirectoryTree ERROR: LIST Path '&1' not found or not writeable", entry( minWhich, chListDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "CompileDirectoryTree: Unable to initialize." ).  
      end.     
      
      file-info:file-name = entry( minWhich, chDebugDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("CompileDirectoryTree ERROR: DEBUG Path '&1' not found or not writeable", entry( minWhich, chDebugDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "CompileDirectoryTree: Unable to initialize." ).  
      end.   
    
    end. /* do minWhich */
    
    obReport:WriteMessage ( "Start CompileDirectoryTree" ).
    
  end method.

  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in launcher */
    super:Process().

    CompileAll().
  end method.
  
  method private void CompileAll():
    define buffer bfContents for ttContents.

    for each bfContents where not bfContents.lgIsProcessed and not lgIsDir:
      CompileOne( 
          bfContents.chSourceDirectory,
          bfContents.chName,
          bfContents.chFullPath,
          bfContents.chPackage,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.
  
  method private void CompileOne(
      ipchSourceDirectory as character,
      ipchName as character,
      ipchFullPath as character,
      ipchPackage as character,
      ipchExtension as character
      ):
    define variable mchRunTarget as character no-undo.
    define variable mchListTarget as character no-undo.
    define variable mchXrefTarget as character no-undo.
    define variable mchDebugTarget as character no-undo.
    define variable minError as integer no-undo.
    define variable minWhich as integer no-undo.
    define variable mchType as character no-undo initial "Error,Warning".
    
    minWhich = lookup( ipchSourceDirectory, chSourceDirectoryList ).
    if minWhich = 0
    then do:
      obReport:WriteMessage ( substitute("CompileDirectoryTree: Source direectory &1 not in Source Directory List for &2", ipchSourceDirectory, ipchFullPath ) ).
      inTotalExceptions = inTotalExceptions + 1.
      return.
    end.
    
    if ipchPackage > ""
    then do:
      assign
          mchRunTarget = entry( minWhich, chRunDirectoryList ) + chSeparator + ipchPackage
          mchListTarget = entry( minWhich, chListDirectoryList ) + chSeparator + ipchPackage
          mchXrefTarget = entry( minWhich, chXrefDirectoryList ) + chSeparator + ipchPackage
          mchDebugTarget = entry( minWhich, chDebugDirectoryList ) + chSeparator + ipchPackage
          .
    end.
    else do:
      assign
          mchRunTarget = entry( minWhich, chRunDirectoryList )
          mchListTarget = entry( minWhich, chListDirectoryList )
          mchXrefTarget = entry( minWhich, chXrefDirectoryList )
          mchDebugTarget = entry( minWhich, chDebugDirectoryList )
          .      
    end.
    do on error undo, throw:
      obPathUtilities:MakeDirectory (mchRunTarget).
      obPathUtilities:MakeDirectory (mchListTarget).
      obPathUtilities:MakeDirectory (mchXrefTarget).
      obPathUtilities:MakeDirectory (mchDebugTarget).
      inTotalCompiles = inTotalCompiles + 1.
      catch eobAppError as Progress.Lang.AppError:
        undo, throw new Progress.Lang.AppError( "Aborting CompileDirectoryTree." ).
      end catch.
    end. 
        
    if ipchExtension = "cls"
    then do:
      compile value(ipchFullPath)
          save into value(entry( minWhich, chRunDirectoryList) )
          listing value(obPathUtilities:ListFile( entry( minWhich, chListDirectoryList), ipchPackage, ipchName ))
          page-size 127 page-width 255
          xref-xml value(mchXrefTarget)
          debug-list value(obPathUtilities:DebugFile( entry( minWhich, chDebugDirectoryList ), ipchPackage, ipchName ))
          v6frame use-revvideo
          no-error.
    end.
    else do:
      compile value(ipchFullPath)
          save into value(mchRunTarget)
          listing value(obPathUtilities:ListFile( entry( minWhich, chListDirectoryList ), ipchPackage, ipchName ))
          page-size 127 page-width 255
          xref-xml value(mchXrefTarget)
          debug-list value(obPathUtilities:DebugFile( entry( minWhich, chDebugDirectoryList), ipchPackage, ipchName ))
          v6frame use-revvideo
          no-error.

    end.    

    if compiler:error then inTotalErrors = inTotalErrors + 1.
    if     ( lgReportErrors and compiler:error )
        or ( lgReportWarnings and compiler:warning )
    then do:
      obReport:WriteMessage ( substitute("CompileDirectoryTree: Compile error for &1", ipchFullPath ) ).
      inTotalExceptions = inTotalExceptions + 1.
      if compiler:num-messages > 0
      then do minError = 1 to compiler:num-messages:
        if     ( compiler:get-message-type(minError) = 1 and lgReportErrors ) 
            or ( compiler:get-message-type(minError) = 2 and lgReportWarnings )
        then do: 
          obReport:WriteMessage (  substitute("CompileDirectoryTree: Compile &1: &2", entry(compiler:get-message-type(minError), mchType), compiler:get-message(minError)) ).
          inTotalExceptions = inTotalExceptions + 1.
        end.
      end.      
    end.

  end method.
  
end class.