/*------------------------------------------------------------------------
    File        : Driver
    Purpose     : 
    Syntax      : 
    Description : Controls the processing of the ABL2DB components.
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sat Mar 22 10:41:43 CDT 2014
    Notes       : Use Process for stepwise progression of all standard functions
                : Use ProcessX individually for special purposes
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.*.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.

routine-level on error undo, throw.

class com.cintegrity.ABL2DB.Driver: 
  define public property chDataBaseList as character no-undo get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chDataBaseConnectParams as character no-undo get. set.
  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaFileList as character no-undo get. set.
  define public property chSourceDirectoryList as character no-undo get. set.
  define public property chCompilableExtensions as character no-undo get. set.
  define public property chSourceExtensions as character no-undo get. set.
  define public property lgExcludeNoExtension as logical no-undo get. set.
  define public property chExcludeDirectories as character no-undo get. set.
  define public property lgIncludeEmptyDirectories as logical no-undo get. set.
  define public property chRunDirectoryList as character no-undo get. set.
  define public property chXrefDirectoryList as character no-undo get. set.
  define public property chListDirectoryList as character no-undo get. set.
  define public property chDebugDirectoryList as character no-undo get. set. 
  define public property chReportDestination as character no-undo get. set.
  define public property chPropath as character no-undo get. set.
  define public property lgReportErrors as logical no-undo initial true get. set.
  define public property lgReportWarnings as logical no-undo initial true get. set.
  define public property chMenuFile as character no-undo get. set.
  define public property chMenuItemFile as character no-undo get. set.
  define public property chFunctionalUnitFile as character no-undo get. set.
  
  define private variable chSeparator as character no-undo.
  define private variable chAntiSeparator as character no-undo.

	constructor public Driver (  ):
		
	end constructor.

	destructor public Driver ( ):
    DetachDatabases().
	end destructor.

  method public void Initialize():
    define variable minWhich as integer no-undo.
    define variable mchEntry as character no-undo.
    
    if num-entries(chSourceDirectoryList, "/") > num-entries(chSourceDirectoryList, "~\")
    then 
      assign
          chSeparator = "/"
          chAntiSeparator = "~\"
          .
    else 
      assign
          chSeparator = "~\"
          chAntiSeparator = "/"
          .
    if chPropath > "" then propath = chPropath + "," + propath.
    
    if opsys = "win32"
    then do minWhich = 1 to num-entries(propath, "," ):
      mchEntry = entry( minWhich, propath, "," ).
      if mchEntry = "."
      then log-manager:write-message ( "Driver: Do you *really* need Current Directory ['.'] in the Propath.  Might cause trouble." ).
      else if substring( mchEntry ,2 ,1 ) <> ":"
      then do:
        log-manager:write-message (substitute("Driver: Propath entry &1 does not contain required drive letter", mchEntry )).
        undo, throw new Progress.Lang.AppError( "Aborting." ).         
      end.
    end.

    AttachDatabases().
 
  end method.
  
  method public void Process ( ):
    define variable minWhich as integer no-undo.
    /* Uncomment TestBuildDirectoryTree and comment out everything else to test list of files
    TestBuildDirectoryTree().
    */ 
    ProcessCompile().

    ProcessSchema().

    ProcessDiskFiles().

    ProcessSubUnits().

    ProcessBlocks().

    ProcessShared().

    ProcessRunLinks().

    ProcessTableLinks().

    GetDescriptions().

    BuildMenu().
  
    catch eobSysError as Progress.Lang.SysError :
      log-manager:write-message(substitute("Unexpected Exception: &1", eobSysError:getMessage(1)), "SysERROR").
      if eobSysError:NumMessages > 1
      then do minWhich = 2 to eobSysError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(minWhich)), "SysERROR").
      end.
      do minWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
        log-manager:write-message(entry(minWhich,eobSysError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobSysError no-error.
    end catch.
    
    catch eobOtherError as Progress.Lang.Error :
      log-manager:write-message(substitute("Other error: &2", eobOtherError:getMessage(1))).  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobOtherError:getMessage(minWhich)), "OtherERROR").
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        log-manager:write-message(entry(minWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobOtherError no-error.
    end catch.
  end method.

  method public void TestBuildDirectoryTree ():
    define variable mobBuildDirectoryTree as class BuildDirectoryTree no-undo.
    define variable minWhich as integer no-undo.
    
    mobBuildDirectoryTree = new BuildDirectoryTree( chSourceDirectoryList ).
    
    mobBuildDirectoryTree:chReportDestination = "".
    mobBuildDirectoryTree:chFileListDestination = chReportDestination + chSeparator + "TestBuildDirectoryTree.txt".
    mobBuildDirectoryTree:chIncludeExtensions = chSourceExtensions.
    mobBuildDirectoryTree:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildDirectoryTree:chExcludeDirectories = chExcludeDirectories.
    
    mobBuildDirectoryTree:Initialize().
    mobBuildDirectoryTree:Process().
    mobBuildDirectoryTree:FileList().

    delete object mobBuildDirectoryTree.
  end method.
  

  method public void ProcessCompile ():
    define variable mobCompileDirectoryTree as class CompileDirectoryTree no-undo.

    mobCompileDirectoryTree = new CompileDirectoryTree( chSourceDirectoryList ).
    
    mobCompileDirectoryTree:chRunDirectoryList = chRunDirectoryList.
    mobCompileDirectoryTree:chListDirectoryList = chListDirectoryList.
    mobCompileDirectoryTree:chXrefDirectoryList = chXrefDirectoryList.
    mobCompileDirectoryTree:chDebugDirectoryList = chDebugDirectoryList.
    mobCompileDirectoryTree:chReportDestination = chReportDestination + chSeparator + "CompileExceptions.txt".
    mobCompileDirectoryTree:chPropath = chPropath.
    mobCompileDirectoryTree:chIncludeExtensions = chCompilableExtensions.
    mobCompileDirectoryTree:lgExcludeNoExtension = lgExcludeNoExtension.
    mobCompileDirectoryTree:chExcludeDirectories = chExcludeDirectories.
    mobCompileDirectoryTree:lgExcludeEncrypted = false.
    
    mobCompileDirectoryTree:Initialize().
    mobCompileDirectoryTree:Process().

    delete object mobCompileDirectoryTree.
  end method.

  method public void ProcessSchema ():
    /* Consider providing an alternate which works directly from the metaschema */
    define variable mobBuildSchemaDF as class BuildSchemaDF no-undo.  

    mobBuildSchemaDF = new BuildSchemaDF( ).
    mobBuildSchemaDF:chDataBaseList        = chDataBaseList.            
    mobBuildSchemaDF:chLogicalDataBaseList = chLogicalDataBaseList.
    mobBuildSchemaDF:chSchemaDirectory     = chSchemaDirectory.    
    mobBuildSchemaDF:chSchemaFileList      = chSchemaFileList.     
    mobBuildSchemaDF:chReportDestination = chReportDestination + chSeparator + "BuildSchemaDFExceptions.txt".
    
    mobBuildSchemaDF:Initialize().
    mobBuildSchemaDF:Process().
    
    delete object mobBuildSchemaDF.   

  end method.

  method public void ProcessDiskFiles ():
    define variable mobBuildDiskFiles as class BuildDiskFiles no-undo.
    
    mobBuildDiskFiles = new BuildDiskFiles ( chSourceDirectoryList ).
    
    mobBuildDiskFiles:chRunDirectoryList = chRunDirectoryList.
    mobBuildDiskFiles:chListDirectoryList = chListDirectoryList.
    mobBuildDiskFiles:chXrefDirectoryList = chXrefDirectoryList.
    mobBuildDiskFiles:chDebugDirectoryList = chDebugDirectoryList.
    mobBuildDiskFiles:chReportDestination = chReportDestination + chSeparator + "BuildDiskFilesExceptions.txt".
    mobBuildDiskFiles:chIncludeExtensions = chSourceExtensions.
    mobBuildDiskFiles:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildDiskFiles:chExcludeDirectories = chExcludeDirectories.
    mobBuildDiskFiles:lgExcludeEncrypted = false.
    
    mobBuildDiskFiles:Initialize().
    mobBuildDiskFiles:Process().
    
    delete object mobBuildDiskFiles.
  end method.

  method public void ProcessSubUnits ():
    define variable mobBuildSubUnits as class BuildSubUnits no-undo.
    
    mobBuildSubUnits = new BuildSubUnits ( chSourceDirectoryList ).
    
    mobBuildSubUnits:chIncludeExtensions = chCompilableExtensions.
    mobBuildSubUnits:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildSubUnits:chExcludeDirectories = chExcludeDirectories.
    mobBuildSubUnits:chReportDestination = chReportDestination + chSeparator + "BuildSubUnitsExceptions.txt".
    
    mobBuildSubUnits:Initialize().
    mobBuildSubUnits:Process().
        
    delete object mobBuildSubUnits.
  end method.
 
  method public void ProcessBlocks ():
    define variable mobBuildBlocks as class BuildBlocks no-undo.
    
    mobBuildBlocks = new BuildBlocks ( chSourceDirectoryList ).

    mobBuildBlocks:chIncludeExtensions = chCompilableExtensions.
    mobBuildBlocks:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildBlocks:chExcludeDirectories = chExcludeDirectories.
    mobBuildBlocks:chReportDestination = chReportDestination + chSeparator + "BuildBlocksExceptions.txt".
    mobBuildBlocks:chLogicalDataBaseList = chLogicalDataBaseList.
    mobBuildBlocks:chAliasList = chAliasList.
    mobBuildBlocks:chSchemaDirectory = chSchemaDirectory.
    
    mobBuildBlocks:Initialize().
    mobBuildBlocks:Process().
    
    delete object mobBuildBlocks.    
  end method.

 method public void ProcessShared ():
    define variable mobBuildShared as class BuildShared no-undo.
    
    mobBuildShared = new BuildShared ( chSourceDirectoryList ).

    mobBuildShared:chIncludeExtensions = chCompilableExtensions.
    mobBuildShared:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildShared:chExcludeDirectories = chExcludeDirectories.
    mobBuildShared:chReportDestination = chReportDestination + chSeparator + "BuildSharedExceptions.txt".
    mobBuildShared:chLogicalDataBaseList = chLogicalDataBaseList.
    mobBuildShared:chAliasList = chAliasList.
    mobBuildShared:chSchemaDirectory = chSchemaDirectory.
        
    mobBuildShared:Initialize().
    mobBuildShared:Process().
    
    delete object mobBuildShared.    
  end method.
  
  method public void ProcessRunLinks ():
    define variable mobBuildRunLinks as class BuildRunLinks no-undo.
    
    mobBuildRunLinks = new BuildRunLinks ( chSourceDirectoryList ).
    
    mobBuildRunLinks:chIncludeExtensions = chCompilableExtensions.
    mobBuildRunLinks:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildRunLinks:chExcludeDirectories = chExcludeDirectories.
    mobBuildRunLinks:chReportDestination = chReportDestination + chSeparator + "BuildRunLinksExceptions.txt".
    
    mobBuildRunLinks:Initialize().
    mobBuildRunLinks:Process().
        
    delete object mobBuildRunLinks.
  end method.

  method public void ProcessTableLinks ():
    define variable mobBuildTableLinks as class BuildTableLinks no-undo.
    
    mobBuildTableLinks = new BuildTableLinks ( chSourceDirectoryList ).
    
    mobBuildTableLinks:chIncludeExtensions = chCompilableExtensions.
    mobBuildTableLinks:lgExcludeNoExtension = lgExcludeNoExtension.
    mobBuildTableLinks:chExcludeDirectories = chExcludeDirectories.
    mobBuildTableLinks:chReportDestination = chReportDestination + chSeparator + "BuildTableLinksExceptions.txt".
    
    mobBuildTableLinks:Initialize().
    mobBuildTableLinks:Process().
        
    delete object mobBuildTableLinks.
  end method.
   
  method public void GetDescriptions ():
    define variable mobGetDescriptions as class GetDescriptions no-undo.
    
    mobGetDescriptions = new GetDescriptions ( chSourceDirectoryList ).
    
    mobGetDescriptions:chIncludeExtensions = chSourceExtensions.
    mobGetDescriptions:lgExcludeNoExtension = lgExcludeNoExtension.
    mobGetDescriptions:chExcludeDirectories = chExcludeDirectories.
    mobGetDescriptions:chReportDestination = chReportDestination + chSeparator + "GetDescriptionExceptions.txt".
    
    mobGetDescriptions:Initialize().
    mobGetDescriptions:Process().
    
    delete object mobGetDescriptions.    
  end method.
  
  method public void BuildMenu ():
    define variable mobBuildMenu as class BuildMenu no-undo.
    
    mobBuildMenu = new BuildMenu ( ).
    
    mobBuildMenu:chSourceDirectoryList = chSourceDirectoryList.
    mobBuildMenu:chMenuFile = chMenuFile.
    mobBuildMenu:chMenuItemFile = chMenuItemFile.
    mobBuildMenu:chFunctionalUnitFile = chFunctionalUnitFile.
    mobBuildMenu:chReportDestination = chReportDestination + chSeparator + "BuildMenuExceptions.txt".
    
    mobBuildMenu:Initialize().
    mobBuildMenu:Process().
    
    delete object mobBuildMenu.
  end method.
  
  method private void AttachDatabases():
    
    define variable minWhich as integer no-undo.
    define variable mchCurrentDB as character no-undo.
    define variable mchLogicalDB as character no-undo.
    
    do minWhich = 1 to num-entries( chDataBaseList ):
      mchCurrentDB = entry( minWhich, chDataBaseList ).
      mchLogicalDB = entry( minWhich, chLogicalDataBaseList ).
      connect value( mchCurrentDB ) value( entry( minWhich, chDataBaseConnectParams, "|" )).
      if not connected( mchLogicalDB )
      then do:
        log-manager:write-message(substitute("Database &1 (&2) did not connect", mchLogicalDB, mchCurrentDB )).
        undo, throw new Progress.Lang.AppError( "Failed to connect to needed database(s)").
      end.
      if chAliasList > "" and entry( minWhich, chAliasList ) <> ""
      then create alias value( entry( minWhich, chAliasList ) ) for database value( mchLogicalDB ).
    end.

    catch eobSysError as Progress.Lang.SysError :
      define variable inWhich as integer no-undo.
      log-manager:write-message(substitute("Driver: Unexpected Exception attaching database &1: &2", mchCurrentDB, eobSysError:getMessage(1)), "SYSERROR").
      if eobSysError:NumMessages > 1
      then do inWhich = 2 to eobSysError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(inWhich)), "SYSERROR").
      end.
    end catch.
    
  end method.
  
  method private void DetachDatabases().
  
    define variable minWhich as integer no-undo.
      
    do minWhich = 1 to num-entries( chDataBaseList ):
      disconnect value( entry( minWhich, chDataBaseList )) no-error.
    end.  
  
  end method. 

end class.