 /*------------------------------------------------------------------------
    File        : GetDescriptions
    Purpose     : 
    Syntax      : 
    Description : Scan files and extract a description
    Author(s)   : Thomas Mercer-Hursh
    Created     : Thu May 08 16:44:34 CDT 2014
    Notes       : Needs to be customized to code base.
                  For any code base where the desired description is 
                  found at the top of the source file in some form,
                  just customize the indicated part of GetDescription().
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.

routine-level on error undo, throw.

class com.cintegrity.ABL2DB.GetDescriptions inherits BuildDirectoryTree: 
  define public property inTotalFiles as integer no-undo get. private set.
  define public property inTotalExceptions as integer no-undo get. private set.
  
	constructor public GetDescriptions (
      ipchSourceDirectoryectory as character
      ):  
    super ( ipchSourceDirectoryectory ).
	end constructor.

	destructor public GetDescriptions ( ):
    if valid-object(obReport) 
    then obReport:WriteMessage ( substitute("End of scan with &1 total files", inTotalFiles ) ).
    log-manager:write-message ( substitute( "End of GetDescriptions with &1 exceptions", inTotalExceptions )).
	end destructor.

  method public override void Initialize():
    super:Initialize().
  
    log-manager:write-message ( "Start of GetDescriptions").
    obReport:WriteMessage ( substitute("Exception list for get description build of &1", chSourceDirectoryList ) ).
    
  end method.

  method public override void Process():
    /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in Driver */
    super:Process().
    
    ScanAll ().
    
  end method. 

  method private void ScanAll():
    define buffer bfContents for ttContents.
    
    for each bfContents where not bfContents.lgIsProcessed and not lgIsDir:
      ScanOne( 
          bfContents.chName,
          bfContents.chPackage,
          bfContents.chSourceDirectory,
          bfContents.chFullPath,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */
  end method.
  
  method private void ScanOne (
      ipchName as character,
      ipchPackage as character,
      ipchSourceDirectory as character,
      ipchFullPath as character,
      ipchExtension as character
      ):
    define variable mchDescription as character no-undo.
    
    define buffer bfDiskFile for DiskFile.
    define buffer bfCompileUnit for CompileUnit.
        
    mchDescription = ExtractDescription ( ipchFullPath ).
    inTotalFiles = inTotalFiles + 1.
    
    if mchDescription > ""
    then do for bfDiskFile:
      find bfDiskFile exclusive-lock
          where bfDiskFile.chBaseDirectory = ipchSourceDirectory
          and   bfDiskFile.chPackage = ipchPackage
          and   bfDiskFile.chName = ipchName
          no-error.
      if not available( bfDiskFile )
      then do:
        obReport:WriteMessage ( substitute("GetDescription: DiskFile for &1 is missing", ipchFullPath ) ).
        inTotalExceptions = inTotalExceptions + 1.
      end. 
      else bfDiskFile.chDescription = mchDescription.
    end.    
  end method.
  
  method public character ExtractDescription (
      ipchSourceFile as character
      ):
    define variable mlcFile as longchar no-undo.
    define variable mchDescription as character no-undo.
    
    file-info:file-name = ipchSourceFile.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      obReport:WriteMessage (substitute("GetDescription: Source file '&1' not found or not readable", ipchSourceFile )).
      inTotalExceptions = inTotalExceptions + 1.
    end.
    else do:
      copy-lob from file ipchSourceFile to mlcFile.
      /* Customize here to extract the desired description, e.g., search top lines for Purpose: and/or
         Description: or whatever is appropriate in the local code base.  What follows is for the 
         Varnet standard in which the top line of the file is a comment giving the program name
         followed by a title
      */ 
      mchDescription = entry( 1,  mlcFile, "~n" ).
      if index(mchDescription, "/*" ) > 0
      then do:
        mchDescription = trim(replace(replace(mchDescription,"*/",""), "/*", "")). 
        if num-entries(mchDescription," ") > 1
        then mchDescription = substring(mchDescription,length(entry(1, mchDescription, " ")) + 1).
        /* End of customization section */
      end.
      else mchDescription = "".
      return mchDescription.
    end.
  end method.
    
end class.