/*------------------------------------------------------------------------
    File        : ObjectXrefHandler.p
    Purpose     : 
    Syntax      :
    Description : Provides handler for SAX Reader (should be .cls!)
    Author(s)   : 
    Created     : Mon Mar 10 15:38:04 CDT 2014
    Notes       :
  ----------------------------------------------------------------------*/

block-level on error undo, throw.

define variable chCompileUnitID as character no-undo.
define variable chCompileUnitName as character no-undo.
define variable chCurrentCompileSubUnitID as character no-undo.
define variable chCurrentCompileSubUnitName as character no-undo.
define variable chCurrentNodeType as character no-undo.
define variable chCurrentModeValue as character no-undo.
define variable chLineNo as character no-undo.
define variable inTotalIPs as integer no-undo.
define variable inTotalMethods as integer no-undo.
define variable inTotalFunctions as integer no-undo.

procedure Initialize:
  define input parameter ipchCompileUnitID as character no-undo.

  define buffer bfCompileUnit for CompileUnit.
  define buffer bfDiskFile for DiskFile.
  
  chCompileUnitID = ipchCompileUnitID.

  find bfCompileUnit no-lock where bfCompileUnit.chID = chCompileUnitID no-error.
  if not available bfCompileUnit
  then do:
    log-manager:write-message (substitute("ObjectXrefHandler.p: CompileUnit (ID = &1) not found in Initialize.", chCompileUnitID)).
    undo, throw new Progress.Lang.AppError( "ObjectXrefHandler.p: Aborting." ).   
  end.
  find bfDiskFile no-lock where bfDiskFile.chID = bfCompileUnit.chDiskFileID no-error.
  if not available bfDiskFile
  then do:
    log-manager:write-message (substitute("ObjectXrefHandler.p: DiskFile for CompileUnit (ID = &1) not found in Initialize.", chCompileUnitID)).
    undo, throw new Progress.Lang.AppError( "ObjectXrefHandler.p: Aborting." ).   
  end.  
  chCompileUnitName = bfDiskFile.chName.
  
  assign
      inTotalIPs = 0
      inTotalMethods = 0
      inTotalFunctions = 0
      .
 
end procedure.

procedure StartElement:
    define input  parameter ipchNameSpaceURI as character  no-undo.
    define input  parameter ipchLocalName as character  no-undo.
    define input  parameter ipchElementType as character  no-undo.
    define input  parameter iphnAttributes as handle no-undo.
  
  define variable lchReferenceType as character no-undo.
  define variable lchIdentifier as character no-undo.
    
  define buffer bfCompileSubUnit for CompileSubUnit.
    
  case ipchElementType:
    when "REFERENCE"                           /* Everything is a reference */
      then do:
        lchReferenceType = iphnAttributes:get-value-by-qname("Reference-type").
        lchIdentifier = iphnAttributes:get-value-by-qname("Object-identifier").
        case lchReferenceType:
          when "INCLUDE"                                         /* include files */ 
            then do:
              /* Need to do in link pass because IncludeFile may not yet have been created */
            end.
          when "FUNCTION"                                             /* Function */
            then do for bfCompileSubUnit:
              create bfCompileSubUnit.
              assign
                  bfCompileSubUnit.chID = guid
                  bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                  bfCompileSubUnit.chName = lchIdentifier
                  bfCompileSubUnit.chType = "Function"
                  bfCompileSubUnit.lgIsPrivate = no
                  bfCompileSubUnit.lgIsProtected = no
                  chCurrentCompileSubUnitName = lchIdentifier
                  chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                  inTotalFunctions = inTotalFunctions + 1
                  .
            end.
          when "PRIVATE-FUNCTION"                             /* Private Function */
            then do for bfCompileSubUnit:
              create bfCompileSubUnit.
              assign
                  bfCompileSubUnit.chID = guid
                  bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                  bfCompileSubUnit.chName = lchIdentifier
                  bfCompileSubUnit.chType = "Function"
                  bfCompileSubUnit.lgIsPrivate = yes
                  bfCompileSubUnit.lgIsProtected = no
                  chCurrentCompileSubUnitName = lchIdentifier
                  chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                  inTotalFunctions = inTotalFunctions + 1
                  .
            end.
          when "PROCEDURE"                                           /* Procedure */
            then do for bfCompileSubUnit:
              create bfCompileSubUnit.
              assign
                  bfCompileSubUnit.chID = guid
                  bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                  bfCompileSubUnit.chName = lchIdentifier
                  bfCompileSubUnit.chType = "IP"
                  bfCompileSubUnit.lgIsPrivate = no
                  bfCompileSubUnit.lgIsProtected = no
                  chCurrentCompileSubUnitName = lchIdentifier
                  chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                  inTotalIPs = inTotalIPs + 1
                  .
            end.
          when "PRIVATE-PROCEDURE"                           /* Private Procedure */
            then do for bfCompileSubUnit:
              create bfCompileSubUnit.
              assign
                  bfCompileSubUnit.chID = guid
                  bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                  bfCompileSubUnit.chName = lchIdentifier
                  bfCompileSubUnit.chType = "IP"
                  bfCompileSubUnit.lgIsPrivate = yes
                  bfCompileSubUnit.lgIsProtected = no
                  chCurrentCompileSubUnitName = lchIdentifier
                  chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                  inTotalIPs = inTotalIPs + 1
                  .
            end.   
          when "METHOD"                                                 /* Method */
            then do for bfCompileSubUnit:
              create bfCompileSubUnit.
              assign
                  bfCompileSubUnit.chID = guid
                  bfCompileSubUnit.chCompileUnitID = chCompileUnitID
                  bfCompileSubUnit.chName = lchIdentifier
                  bfCompileSubUnit.chType = "Method"
                  bfCompileSubUnit.lgIsPrivate = no
                  bfCompileSubUnit.lgIsProtected = no
                  chCurrentCompileSubUnitName = lchIdentifier
                  chCurrentCompileSubUnitID = bfCompileSubUnit.chID
                  inTotalMethods = inTotalMethods + 1
                  .             
            end.                  
          when "Access-mode"
            then do:
              chCurrentNodeType = "Access-mode".
              chCurrentModeValue = "".
            end.
          when "Line-num"
            then do:
              chCurrentNodeType = "Line-num".
              chLineNo = "".
            end.
          otherwise do:
            /* Nothing? */
          end.
        end. /* case ReferenceType */
      end. /* when Reference */
  end. /* case ipchElementType */
end procedure.

procedure Characters:
  define input parameter ipchData as memptr no-undo.
  define input parameter ipinNumChars as integer no-undo.

  case chCurrentNodeType:
    when "Access-mode" then chCurrentModeValue = chCurrentModeValue + GET-STRING(ipchData, 1, get-size(ipchData)). 
    when "Line-num" then chLineNo = chLineNo + GET-STRING(ipchData, 1, get-size(ipchData)). 
  end case.

end procedure.

procedure EndElement:
  define input  parameter ipchNameSpaceURI as character  no-undo.
  define input  parameter ipchLocalName as character  no-undo.
  define input  parameter ipchElementType as character  no-undo.

  define buffer bfCompileSubUnit for CompileSubUnit.
  
  case ipchElementType:
    when "INCLUDE"                                         /* Include files */ 
      then do:
        /* Need to do in link pass because IncludeFile may not yet have been created */        
      end.
    when "FUNCTION"                                             /* Function */
      then do:
        run UpdateLineNo.
      end.
    when "PRIVATE-FUNCTION"                             /* Private Function */
      then do:
        run UpdateLineNo.
      end.
    when "PROCEDURE"                                           /* Procedure */
      then do:
        run UpdateLineNo.
      end.
    when "PRIVATE-PROCEDURE"                           /* Private Procedure */
      then do:
        run UpdateLineNo.
      end.      
    when "METHOD"                                      /* Private Procedure */
      then do:
        run UpdateLineNo.
      end.        
    when "Access-mode"
      then do :
        if chCurrentCompileSubUnitName > ""
        then do for bfCompileSubUnit:
          find bfCompileSubUnit exclusive-lock where bfCompileSubUnit.chID = chCurrentCompileSubUnitID no-error.
          if not available bfCompileSubUnit
          then do:
            log-manager:write-message (substitute("ObjectXrefHandler.p: CompileSubUnit not found in Access-Mode for &1 of &2", chCompileUnitName, chCurrentCompileSubUnitName)).
            /* undo, throw new Progress.Lang.AppError( "ObjectXrefHandler.p: Aborting." ).   Non-fatal for now */   
          end.   
          else do:
            case chCurrentModeValue:
              when ""
                then do:
                  bfCompileSubUnit.lgIsPrivate = no.
                  bfCompileSubUnit.lgIsProtected = no.
                end.
              when "PUBLIC"
                then do:
                  bfCompileSubUnit.lgIsPrivate = no.
                  bfCompileSubUnit.lgIsProtected = no.
                end.
              when "PRIVATE"
                then do:
                  bfCompileSubUnit.lgIsPrivate = yes.
                  bfCompileSubUnit.lgIsProtected = no.
                end.
              when "PROTECTED"
                then do:
                  bfCompileSubUnit.lgIsPrivate = no.
                  bfCompileSubUnit.lgIsProtected = yes.
                end.
            end case.      
          end. /* else */
        end. /* CCSU > "" */
      end. /* when "Access-mode" */
    otherwise do:
      /* Nothing? */
    end.
  end.
end procedure.   

procedure UpdateLineNo:
  define variable inLineNo as integer no-undo.
  define buffer bfCompileSubUnit for CompileSubUnit.
  
  find bfCompileSubUnit exclusive-lock where bfCompileSubUnit.chID = chCurrentCompileSubUnitID no-error.
  if not available bfCompileSubUnit
  then do:
    log-manager:write-message (substitute("ObjectXrefHandler.p: CompileSubUnit not found in UpdateLineNo for &1 of &2", chCompileUnitName, chCurrentCompileSubUnitName)).
    /* undo, throw new Progress.Lang.AppError( "ObjectXrefHandler.p: Aborting." ).   Non-fatal for now */   
  end.   
  else do:
    assign inLineNo = integer(chLineNo) no-error.
    if error-status:error
    then do:
      log-manager:write-message (substitute("ObjectXrefHandler.p: Error converting line no = '&1' in UpdateLineNo for &2 of &3", chLineNo, chCompileUnitName, chCurrentCompileSubUnitName)).
      /* undo, throw new Progress.Lang.AppError( "ObjectXrefHandler.p: Aborting." ).   Non-fatal for now */   
    end.
    else bfCompileSubUnit.inLineNo = inLineNo.      
  end. 
end procedure.     

function TotalIPs returns integer:
  return inTotalIPs.
end function.

function TotalMethods returns integer:
  return inTotalMethods.
end function.

function TotalFunctions returns integer:
  return inTotalFunctions.
end function.