 /*------------------------------------------------------------------------
    File        : FileType
    Purpose     : 
    Syntax      : 
    Description : Utilities for use with program, class, and include names.
    Author(s)   : Thomas Mercer-Hursh
    Created     : Tue Feb 28 15:20:05 PST 2012
    Notes       : Customize to suit code base by setting public properties
                : lgExcludeNoExtension = false is provided for code bases like
                : Varnet which have compile units with no extension.  It applies
                : to both Source and Compile files.
  ----------------------------------------------------------------------*/

using Progress.Lang.*.

routine-level on error undo, throw.

class com.cintegrity.Filesystem.FileType: 
  define public property chSourceExtensions as character no-undo initial "bat,cls,i,inq,f,p,pn,po,px,sh,v,w" get. set.
  define public property chCompilableExtensions as character no-undo initial "p,w,cls" get. set.
  define public property lgExcludeNoExtension as logical no-undo initial true get. set.
  define public property chIncludeExtensions as character no-undo initial "i" get. set.

	constructor public FileType (  ):

  end constructor.

  method public logical IsSourceFile ( input ipchFileName as character ):
    define variable mchExtension as character no-undo.        
    mchExtension = ExtensionFromName( ipchFileName ).
    
    if lookup( mchExtension, chSourceExtensions ) > 0
        or mchExtension = "" and not lgExcludeNoExtension
    then return true.
    else return false.
  end method.

  method public logical IsCompileUnit ( input ipchFileName as character ):
    define variable mchExtension as character no-undo.        
    mchExtension = ExtensionFromName( ipchFileName ).
    
    if lookup( mchExtension, chCompilableExtensions ) > 0
        or mchExtension = "" and not lgExcludeNoExtension
    then return true.
    else return false.
  end method.

  method public logical IsClass ( input ipchFileName as character ):
    if ipchFileName matches "*~~.cls"
    then return true.
    else return false.
  end method.

  method public logical IsInclude ( input ipchFileName as character ):
    define variable mchExtension as character no-undo.        
    mchExtension = ExtensionFromName( ipchFileName ).
    
    if lookup( mchExtension, chIncludeExtensions ) > 0
    then return true.
    else return false.
  end method.
                                                         /* Obtain Extension */
  method public character ExtensionFromName (
      ipchName as character
      ):
    define variable mchExtension as character no-undo.    
       
    if num-entries(ipchName, ".") > 1 
    then mchExtension = entry(num-entries(ipchName, "."), ipchName, "." ).
    return mchExtension.    
  end.

end class.