/*------------------------------------------------------------------------
   File        : FilesInDirectory
   Purpose     : 
   Syntax      : 
   Description : Build a list of files under a directory
   Author(s)   : Thomas Mercer-Hursh
   Created     : Wed Feb 29 15:48:10 PST 2012
   Notes       : 
 ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.Filesystem.FileType.

routine-level on error undo, throw.

class com.cintegrity.Filesystem.FilesInDirectory: 

  define public property IsEmpty as logical no-undo get. private set. 
  define public property Size as integer no-undo get. private set.
  
  define private temp-table ttFilename no-undo
      field inSeq as integer
      field chFilename as character
      index ixBySeq as primary unique inSeq
      .
  define private variable inNext as integer no-undo. 
  define private variable inCurrent as integer no-undo extent. 
  define private variable inMaxIterator as integer no-undo initial 10. /* Increase if necessary */
  define private variable inNextIterator as integer no-undo.
  define protected variable obFileType as class FileType.
    		
  constructor public FilesInDirectory ( ):
    obFileType = new FileType().
  end constructor.

  method public void Initialize ( input ipchDirectory as character ):
    Reset().
    extent ( inCurrent ) = inMaxIterator.    
    
    file-info:file-name = ipchDirectory.

    if  (  file-info:file-type = ?
        or index(file-info:file-type, "D":U) = 0
        ) 
    then return error new Progress.Lang.AppError(substitute("Directory '%1' does not exist",ipchDirectory)).

    RecurseDirectory(file-info:full-pathname).
    
  end method.

  method private void Reset ():
    assign
        inNext = 0
        Size = 0
        IsEmpty = true
        inNextIterator = 0
        .
    extent ( inCurrent ) = ?.     
  end.

  method private void RecurseDirectory ( input ipchDirectory as character ):

    define variable chBaseName   as character no-undo.
    define variable chFullPath   as character no-undo.
    define variable chAttributes as character no-undo.

    file-info:file-name = ipchDirectory.
    input from os-dir (file-info:full-pathname).
    repeat:
      import chBaseName chFullPath chAttributes.
      if chAttributes matches "*D*":U and not(chBaseName=".":U or chBaseName="..":U) 
      then RecurseDirectory(chFullPath).
      else if chAttributes matches "*F*":U 
      then do:
        if IncludeFile( chBaseName )
        then do:
          create ttFilename.
          assign 
              inNext = inNext + 1
              Size = Size + 1
              ttFilename.inSeq = inNext
              ttFilename.chFilename = chFullPath
              isEmpty = false
              .       
        end.     
      end.
    end.
    input close.
  end method.
  
  method protected logical IncludeFile ( input ipchBaseName as character ):
    return true.  /* Override in subclasses to get lists by file types */
  end method.

  method public void Empty(  ):
    empty temp-table ttFilename.
    Reset().
  end method.  
 
  method public integer GetIterator ( ):
    inNextIterator = inNextIterator + 1.
    if inNextIterator > inMaxIterator
    then undo, throw new Progress.Lang.AppError("Maximum Iterators per Collection exceeded for FilesInDirectory").
    return inNextIterator.
  end method. 
  
  method public void ToStart( ipinWhichIterator as integer ):
    inCurrent[ ipinWhichIterator ] = 0.
  end method.

  method public void ToEnd( ipinWhichIterator as integer ):
    if not IsEmpty
    then do:
      find last ttFilename no-lock no-error.
      if available ( ttFilename )
      then inCurrent[ ipinWhichIterator ] = ttFilename.inSeq + 1.
      else undo, throw new Progress.Lang.AppError("Find last by ixBySeq failed in FilesInDirectory").
    end.
    else inCurrent[ ipinWhichIterator ] = 0.
  end method.
    
  method public character GetNext( ipinWhichIterator as integer ):
    if not IsEmpty
    then do:
      find next ttFilename no-lock where ttFilename.inSeq > inCurrent[ ipinWhichIterator ] no-error.
      if available ( ttFilename )
      then do:
        inCurrent[ ipinWhichIterator ] = ttFilename.inSeq.
        return ttFilename.chFilename.
      end.
      ToEnd( ipinWhichIterator).
      return ?.
    end.
      
    inCurrent[ ipinWhichIterator ] = 0.
    return ? .
      
  end method.

  destructor public FilesInDirectory ( ):
    this-object:Empty().
  end destructor.

end class.