/*------------------------------------------------------------------------
    File        : DocSchemaFromDB
    Purpose     : 
    Syntax      : 
    Description : Create proparse.schema for use by Proparse from connected databases
    Author(s)   : Thomas Mercer-Hursh
    Created     : Mon Mar 05 14:53:16 PST 2012
    Notes       : chSchemaDirectory  Directory in which to create proparse.schema (default session:temp-directory)
                : chSchemaDumpFile   Name of schema dump file (default proparse.schema)
                : chDataBaseList     List of database names to create schema from (allows skipping attached DB)
                : Initialize() to prep
                : DumpSchemas() to do work
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.Proparse.DumpOneSchema.
using org.prorefactor.core.schema.Schema from assembly.

routine-level on error undo, throw.

class com.cintegrity.Proparse.DocSchemaFromDB: 

  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character no-undo get. set.
  define public property chDataBaseList as character no-undo get. set.
    
  define stream stPPSchema.
        		
  constructor public DocSchemaFromDB( ):

  end constructor.
  
  destructor public DocSchemaFromDB( ):
  
  end destructor.
  
  method public void Initialize( ):

    if chSchemaDirectory = "" then chSchemaDirectory = session:temp-directory.
    if chSchemaDumpFile = "" then chSchemaDumpFile = "proparse.schema":U.
    chSchemaDumpFile = chSchemaDirectory + "/" + chSchemaDumpFile.

  end method.
  
  method public void DumpSchemas( ):
    define variable mchOriginalDictDB as character no-undo.
    define variable minWhichDB        as integer   no-undo.
    define variable mobMyDump           as class     DumpOneSchema.
    
    mchOriginalDictDB = ldbname("dictdb":U).

    output stream stPPSchema to value ( chSchemaDumpFile ).
    output stream stPPSchema close.

    repeat minWhichDB = 1 to num-dbs:
      if dbtype(minWhichDB) <> "PROGRESS":U then next.
      if lookup(ldbname(minWhichDB), chDataBaseList ) > 0
      then do:
        delete alias DICTDB.
        create alias DICTDB for database value(ldbname(minWhichDB)).
        
        mobMyDump = new DumpOneSchema().
        mobMyDump:DoDump( chSchemaDumpFile , minWhichDB ).
        delete object mobMyDump.
      end.
    end.
    
    delete alias dictdb.
    create alias dictdb for database value(mchOriginalDictDB).

    mobMyDump = new DumpOneSchema().
    mobMyDump:DumpMetaSchema( chSchemaDumpFile ).
    delete object mobMyDump.    
  
  end.  

end class.