 /*------------------------------------------------------------------------
    File        : Proparser
    Purpose     : 
    Syntax      : 
    Description : Proparse utilities for use by other applications
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sat Apr 04 15:57:01 CDT 2015
    Notes       : 
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.LoggingUtils.FlyWeightLogger.
using com.joanju.proparse.Environment from assembly.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.core.schema.Schema from assembly.
using org.prorefactor.core.TreeUtils from assembly.
using org.prorefactor.refactor.RefactorSession from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.

block-level on error undo, throw.

class com.cintegrity.Proparse.Proparser: 
  define public property chSchemaDirectory as character get. set.
  define public property chSchemaFile as character initial "proparse.schema":u get. set.
  define public property chAliasList as character get. set.
  define public property chDataBaseList as character get. set.
  
  define private variable obProparseEnvironment as Environment no-undo.
  define private variable obProparseSchema as class Schema no-undo.
  define private variable obRefactorSession as class RefactorSession no-undo.
  define private variable chCompileUnit as character no-undo.
  define private variable obParseUnit as ParseUnit.
		
	constructor public Proparser():

	end constructor.

  destructor public Proparser ( ):

  end destructor.
	
	method void Initialize():
	  define variable minWhich as integer no-undo.

    obProparseEnvironment = Environment:instance().
    obProparseSchema = Schema:getInstance().
    obRefactorSession = RefactorSession:getInstance().
    
    obRefactorSession:setContextDirName(chSchemaDirectory + "/").
    obProparseEnvironment:configSet("batch-mode", string(session:batch-mode, "true/false")).
    obProparseEnvironment:configSet("opsys", opsys).
    obProparseEnvironment:configSet("propath", propath).
    obProparseEnvironment:configSet("proversion", proversion).
    obProparseEnvironment:configSet("window-system", session:window-system).
    
    obProparseSchema:clear().
    obProparseSchema:loadSchema(chSchemaDirectory + chSchemaFile).

    obProparseSchema:aliasDelete("").
    if chAliasList > ""
    then do minWhich = 1 to num-entries(chAliasList):
      if entry(minWhich, chAliasList) <> "" and entry(minWhich, chAliasList) <> ?
      then obProparseSchema:aliasCreate(entry(minWhich, chAliasList), entry(minWhich, chDataBaseList)).
    end.
    	  
	end.
	
	method void DoParse (
	    input ipchCompileUnit as character
	    ):
    define variable obJavaFile as java.io.File no-undo.	  
    define variable minWhich as integer no-undo.
	  
	  chCompileUnit = ipchCompileUnit.
    obJavaFile = new java.io.File(ipchCompileUnit).
    obParseUnit = new ParseUnit(obJavaFile).
    obParseUnit:treeParser01().
    
    catch eobOtherError as Progress.Lang.Error :
      log-manager:write-message(substitute("Other error in &1: &2", ipchCompileUnit, eobOtherError:getMessage(1))).
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        log-manager:write-message(substitute("Other error (cont.): &1", eobOtherError:getMessage(minWhich)), "OtherERROR").
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        log-manager:write-message(entry(minWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobOtherError no-error. 
    end catch.
	end method.
	
end class.