/*------------------------------------------------------------------------
    File        : ScanOneCU
    Purpose     : Testing that Compile Units can be parsed by Proparse
    Syntax      : ScanOneCU( CompileUnitToScan )
    Description : Runs Proparse on a Compile Unit to ensure that a valid parse tree is generated
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sat Mar 22 10:41:43 CDT 2014
    Notes       : Assumes that proparse.schema has been created in advance
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.*.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.

routine-level on error undo, throw.

class com.cintegrity.Proparse.ScanOneCU: 
  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character initial "proparse.schema":u get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
      
  define private variable chCompileUnitToScan as character no-undo.
  define private variable obSetEnvironment as class SetEnvironment no-undo.
  define private variable obSetSchemaFromFile as class SetSchemaFromFile no-undo.

	constructor public ScanOneCU ( 
	    chCompileUnitToScanToScan as character
	    ):
		chCompileUnitToScan = chCompileUnitToScanToScan.
    file-info:file-name = chCompileUnitToScan.
    if file-info:full-pathname = ? 
        or not file-info:file-type begins "F" 
        or not file-info:file-type matches "*R*"
    then do:
      log-manager:write-message (substitute("ScanOneCU ERROR: Input Program to Scan &1 not found", chCompileUnitToScan )).
      undo, throw new Progress.Lang.AppError( "BuildSchemaDF: Unable to initialize." ).  
    end.		
	end constructor.

	destructor public ScanOneCU ( ):
    if valid-object( obSetEnvironment ) then delete object obSetEnvironment.
    if valid-object( obSetSchemaFromFile ) then delete object obSetSchemaFromFile.
	end destructor.

  method public void Initialize():
                                                          /* Initialize for Proparse */
    if chSchemaDirectory = "" then chSchemaDirectory = session:temp-directory.
    if chSchemaDumpFile = "" then chSchemaDumpFile = "proparse.schema":U.
    chSchemaDumpFile = chSchemaDirectory + "/" + chSchemaDumpFile.

    obSetEnvironment = new SetEnvironment().
    obSetEnvironment:Initialize().
    
    obSetSchemaFromFile = new SetSchemaFromFile().
    obSetSchemaFromFile:chSchemaDirectory = chSchemaDirectory.
    obSetSchemaFromFile:chLogicalDataBaseList = chLogicalDataBaseList.           
    obSetSchemaFromFile:chAliasList = chAliasList.
    obSetSchemaFromFile:lgDeletePPSchemaFile = false.
    obSetSchemaFromFile:Initialize().
 
  end method.
  
  method void DoParse():
    define variable obJavaFile as java.io.File no-undo.   
    define variable obParseUnit as ParseUnit no-undo.
    define variable minWhich as integer no-undo.
    
    obJavaFile = new java.io.File(chCompileUnitToScan).
    obParseUnit = new ParseUnit(obJavaFile).
    obParseUnit:treeParser01().
    
    if NodeTypes:getTypeName(obParseUnit:getTopNode():GetType()) <> "Program_root"
    then log-manager:write-message(substitute("ScanOneCU: TopNode not Program_root in &1", chCompileUnitToScan)).     

    catch eobSysError as Progress.Lang.SysError :
      log-manager:write-message(substitute("ScanOneCU: Unexpected Exception: &1", eobSysError:getMessage(1)), "SysERROR").
      if eobSysError:NumMessages > 1
      then do minWhich = 2 to eobSysError:NumMessages:
        log-manager:write-message(substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(minWhich)), "SysERROR").
      end.
      do minWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
        log-manager:write-message(entry(minWhich,eobSysError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobSysError no-error.
    end catch.
        
    catch eobOtherError as Progress.Lang.Error :
      log-manager:write-message(substitute("ScanOneCU: Other error in &1: &2", chCompileUnitToScan, eobOtherError:getMessage(1))).
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        log-manager:write-message(substitute("Other error (cont.): &1", eobOtherError:getMessage(minWhich)), "OtherERROR").
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        log-manager:write-message(entry(minWhich,eobOtherError:CallStack,"~n"),"CALLSTACK").
      end.
      delete object eobOtherError no-error. 
    end catch.
  end method.

end class.