/*------------------------------------------------------------------------
    File        : SetSchemaFromFile
    Purpose     : 
    Syntax      : 
    Description : Set up schema for use by Proparse using already generated proparse.schema
    Author(s)   : Thomas Mercer-Hursh
    Created     : Mon Mar 05 14:53:16 PST 2012
    Notes       : chSchemaDirectory    Directory in which proparse.schema resides
                : chSchemaDumpFile     Name of proparse.schema file if not default 
                : chLogicalDataBaseList       List of Database names (only required for aliases)
                : chAliasList          List of Alias names (if any)
                : lgDeletePPSchemaFile Flag of whether to delete proparse.schema when done (default false)
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using org.prorefactor.core.schema.Schema from assembly.

routine-level on error undo, throw.

class com.cintegrity.Proparse.SetSchemaFromFile: 

  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character no-undo get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property lgDeletePPSchemaFile as logical no-undo initial false get. set.
  
  define private variable MyProparseSchema as class Schema no-undo.
    
  constructor public SetSchemaFromFile( ):

  end constructor.
  
  destructor public SetSchemaFromFile( ):
    if valid-object(MyProparseSchema) then delete object MyProparseSchema.
  end destructor.
  
  method public void Initialize( ):
    define variable minWhich as integer no-undo.    

    MyProparseSchema = Schema:getInstance().
    MyProparseSchema:clear().
    MyProparseSchema:aliasDelete("").
    
    if chSchemaDirectory = "" then chSchemaDirectory = session:temp-directory.
    if chSchemaDumpFile = "" then chSchemaDumpFile = "proparse.schema":U.
    chSchemaDumpFile = chSchemaDirectory + "/" + chSchemaDumpFile.
    
    file-info:file-name = chSchemaDumpFile.
    if file-info:full-pathname = ?
    then undo, throw new AppError( substitute("Schema file &1 not found in SetSchemaFromFile", chSchemaDumpFile ) ).

    MyProparseSchema:loadSchema(chSchemaDumpFile).
    
    if lgDeletePPSchemaFile then os-delete value(chSchemaDumpFile). 
         
    repeat minWhich = 1 to num-entries(chAliasList):
      if entry( minWhich, chAliasList) > ? and entry( minWhich, chLogicalDataBaseList) > ?  
      then MyProparseSchema:aliasCreate( entry( minWhich, chAliasList), entry( minWhich, chLogicalDataBaseList) ).
    end.
    
  end method.

end class.