/*------------------------------------------------------------------------
   File        : SharedInfoLister
   Purpose     : 
   Syntax      : 
   Description : Export the tokens parse by Proparse.
   Author(s)   : Thomas Mercer-Hursh
   Created     : Mon Mar 05 16:01:05 PST 2012
   Usage       : Set properties needed, Initialize(), DoList()
               : Assumes proparse.schema has been previously generated in chSchemaDirector
   Notes       : Set up variables as follows
               : chSchemaDirectory - Directory containing proparse.schema
               : chSchemaDumpFile - name of file containing schema, (default proparse.schema)
               : chDataBaseList - comma delimited list of logical DB names (not necessarily all connected)
               : chAliasList - comma delimited list of aliases
               : chFileToParse - the name of the file to process
               : chOutputFile -  where to put the listing
               : The following do what their name sounds like.  ShowShared defaults to true and will cause the 
               : section to appear on the report.  ShowNonShared defaults to false.  When true, will also show
               : the non-shared symbols of that type.  There is no option to show non shared only.
               : ShowOther is any symbol not currently shown in another section and defaults true.
               : lgShowSharedTempTables  
               : lgShowNonSharedTempTables 
               : lgShowSharedBuffers 
               : lgShowNonSharedBuffers 
               : lgShowSharedVariables
               : lgShowNonSharedVariables
               : lgShowSharedFrames 
               : lgShowNonSharedFrames 
               : lgShowSharedDatasets  
               : lgShowNonSharedDatasets 
               : lgShowSharedMenus 
               : lgShowNonSharedMenus  
               : lgShowSharedStreams 
               : lgShowNonSharedStreams 
               : lgShowSharedBrowses 
               : lgShowNonSharedBrowses 
               : lgShowSharedQueries  
               : lgShowNonSharedQueries 
               : lgShowOther 
 ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.
using com.joanju.proparse.NodeTypes from assembly.
using com.joanju.proparse.ProParserTokenTypes from assembly.
using org.prorefactor.core.JPNode from assembly.
using org.prorefactor.core.TreeUtils from assembly.
using org.prorefactor.treeparser.Dataset from assembly.
using org.prorefactor.treeparser.ParseUnit from assembly.
using org.prorefactor.treeparser.Query from assembly.
using org.prorefactor.treeparser.Stream from assembly.
using org.prorefactor.treeparser.Symbol from assembly.
using org.prorefactor.treeparser.SymbolScope from assembly.
using org.prorefactor.treeparser.TableBuffer from assembly.
using org.prorefactor.treeparser.Variable from assembly.
using org.prorefactor.widgettypes.Browse from assembly.
using org.prorefactor.widgettypes.Frame from assembly.
using org.prorefactor.widgettypes.Menu from assembly.
using org.prorefactor.widgettypes.MenuItem from assembly.
using org.prorefactor.widgettypes.Submenu from assembly.

block-level on error undo, throw.

class com.cintegrity.Proparse.SharedInfoLister: 
  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character no-undo get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chFileToParse as character no-undo get. set.
  define public property chOutputFile as character no-undo get. set.

  define public property lgShowSharedTempTables as logical no-undo initial true get. set.
  define public property lgShowNonSharedTempTables as logical no-undo initial false get. set.
  define public property lgShowSharedBuffers as logical no-undo initial true get. set.
  define public property lgShowNonSharedBuffers as logical no-undo initial false get. set.
  define public property lgShowSharedVariables as logical no-undo initial true get. set.
  define public property lgShowNonSharedVariables as logical no-undo initial false get. set.
  define public property lgShowSharedFrames as logical no-undo initial true get. set.
  define public property lgShowNonSharedFrames as logical no-undo initial false get. set.
  define public property lgShowSharedDatasets as logical no-undo initial true get. set.
  define public property lgShowNonSharedDatasets as logical no-undo initial false get. set.
  define public property lgShowSharedMenus as logical no-undo initial true get. set.
  define public property lgShowNonSharedMenus as logical no-undo initial false get. set.
  define public property lgShowSharedStreams as logical no-undo initial true get. set.
  define public property lgShowNonSharedStreams as logical no-undo initial false get. set.  
  define public property lgShowSharedBrowses as logical no-undo initial true get. set.
  define public property lgShowNonSharedBrowses as logical no-undo initial false get. set.   
  define public property lgShowSharedQueries as logical no-undo initial true get. set.
  define public property lgShowNonSharedQueries as logical no-undo initial false get. set.  
  define public property lgShowOther as logical no-undo initial true get. set.
  
  define private variable inCurrentIndent as integer no-undo initial 0.
  define private variable inCurrentLine as integer no-undo initial 0.
  define private variable inFirstNode as integer no-undo.
  define private variable inGrandChild as integer no-undo.

  define private variable obSetEnvironment as class SetEnvironment no-undo.
  define private variable obSetSchemaFromFile as class SetSchemaFromFile no-undo.
  define private variable obParseUnit as class ParseUnit no-undo.
  
  define private stream stOutput.
  
  constructor public SharedInfoLister ( ):
    assign chOutputFile = session:temp-directory + "/SharedInfoLister.txt":U.
  end constructor.
  
  destructor public SharedInfoLister ( ):

  end destructor.
  
  method public void Initialize():
    obSetEnvironment = new SetEnvironment().
    obSetEnvironment:Initialize().
    obSetSchemaFromFile = new SetSchemaFromFile().
    obSetSchemaFromFile:chSchemaDirectory = chSchemaDirectory.
    obSetSchemaFromFile:chSchemaDumpFile = chSchemaDumpFile.
    obSetSchemaFromFile:chLogicalDataBaseList = chLogicalDataBaseList.  
    obSetSchemaFromFile:chAliasList = chAliasList.     
    obSetSchemaFromFile:Initialize().
  end method.
  
  method public void DoList():
    define variable mchFullPath as character no-undo.
    define variable mobJavaFile as class java.io.File no-undo.
    define variable mobNode as class JPNode no-undo.
    define variable obParseUnit as class ParseUnit no-undo.
    define variable minWhich as integer no-undo.

    output stream stOutput to value(chOutputFile).
     
    do on error undo, leave:
      file-info:file-name = chFileToParse.
      mchFullPath = file-info:full-pathname.
      if mchFullPath = ?
      then undo, throw new AppError ( substitute( "File to Parse `&1` not found", chFileToParse ) ).
      
      put stream stOutput unformatted substitute( "Parse of &1 at &2", mchFullPath, now ) skip.
      
      mobJavaFile = new java.io.File(mchFullPath).
      obParseUnit = new ParseUnit(mobJavaFile).
      obParseUnit:treeParser01().

      if lgShowSharedTempTables then ListTempWorkTables( obParseUnit ).
      if lgShowSharedBuffers then ListBuffers( obParseUnit ).
      if lgShowSharedVariables then ListVariables( obParseUnit ).
      if lgShowSharedFrames then ListFrames( obParseUnit ).
      if lgShowSharedDatasets then ListDatasets( obParseUnit ).
      if lgShowSharedMenus then ListMenus( obParseUnit ).      
      if lgShowSharedStreams then ListStreams( obParseUnit ).         
      if lgShowSharedBrowses then ListBrowses( obParseUnit ).     
      if lgShowSharedQueries then ListQueries( obParseUnit ).     
      if lgShowOther then ListRest( obParseUnit ).

      catch eobAppError as Progress.Lang.AppError:
        put stream stOutput unformatted substitute("AppError: &1", eobAppError:ReturnValue ) skip.  
        delete object eobAppError.
      end catch.
      
      catch eobOtherError as Progress.Lang.Error :
        put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
        if eobOtherError:NumMessages > 1
        then do minWhich = 2 to eobOtherError:NumMessages:
          put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
        end.
        do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
          put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
        end.
        delete object eobOtherError no-error.
      end catch.
    end. /* do */

    output stream stOutput close.

  end method. /* DoList */  

  method private void ListTempWorkTables(
      ipobParseUnit as class ParseUnit
      ): 
    define variable molBufferList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobTableBuffer as class TableBuffer no-undo.
    define variable mobTable as class org.prorefactor.core.schema.Table no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.
    define variable mobToken as JPNode no-undo.
    define variable mobChildToken as JPNode no-undo.
    define variable mchTokenType as character no-undo.
    define variable mlgIsNew as logical no-undo initial false.
    define variable mlgIsGlobal as logical no-undo initial false.
    define variable mchDataType as character no-undo.
    define variable mchName as character no-undo.
    define variable mchObjectType as character no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    molBufferList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = molBufferList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedBuffers
    then put stream stOutput unformatted "List of Shared Temp/Work-Tables" skip "-------------------------------" skip.
    else put stream stOutput unformatted "List of All Temp/Work-Tables" skip "----------------------------" skip.
   
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).
      
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "BUFFER" then next.
      
      mobTableBuffer = cast( mobSymbol, TableBuffer ).
      mobTable = mobTableBuffer:getTable().
      if mobSymbol:getDefineNode() <> ? then mlgDefined = yes. else mlgDefined = no.
      if mobSymbol:getIndirectDefineIdNode() <> ? then mlgIndirect = yes. else mlgIndirect = no.

      if mlgDefined
      then do:
        mlgIsNew = false.
        mlgIsGlobal = false.
        mobToken = mobSymbol:getDefineNode():firstChild().
        mchTokenType = mobToken:getText().
        if  mchTokenType = "NEW"
        then do:
          mlgIsNew = true.
          mobChildToken = mobToken:firstChild().
          if NodeTypes:getTypeName(mobChildToken:GetType()) = "GLOBAL"
          then do:
            mlgIsGlobal = true.
            mobChildToken = mobChildToken:nextSibling().
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
          end. /* if global */
          else do:
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
            then do:
              mobToken = mobToken:nextSibling().
              mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
            end.
          end. /* not global */
        end. /* new */
        else do:
          if mchTokenType = "SHARED" 
          then do:
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).
          end.
        end.         
        mchObjectType = NodeTypes:getTypeName(mobToken:GetType()).        
      end.

      if not (mchObjectType = "TEMPTABLE" or mchObjectType = "WORKTABLE") then next.

      if mlgDefined or mlgIndirect
      then do:
        if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedTempTables
        then do:
          put stream stOutput unformatted mobSymbol:fullName().
          if mlgDefined then put stream stOutput unformatted " from definition of " mchObjectType.
          if mlgIndirect then put stream stOutput unformatted " from indirect definition of " mchObjectType.
          if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
          if mobSymbol:isExported() and not mlgIsGlobal then put stream stOutput unformatted " - new shared ".
          if mobSymbol:isExported() and mlgIsGlobal then put stream stOutput unformatted " - new global shared ".
          put stream stOutput unformatted skip.
        end.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListTables */  

  method private void ListBuffers(
      ipobParseUnit as class ParseUnit
      ): 
    define variable molBufferList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobTableBuffer as class TableBuffer no-undo.
    define variable mobTable as class org.prorefactor.core.schema.Table no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.
    define variable mobToken as JPNode no-undo.
    define variable mobChildToken as JPNode no-undo.
    define variable mchTokenType as character no-undo.
    define variable mlgIsNew as logical no-undo initial false.
    define variable mlgIsGlobal as logical no-undo initial false.
    define variable mchDataType as character no-undo.
    define variable mchName as character no-undo.
    define variable mchObjectType as character no-undo.
    define variable minStoreType as integer no-undo.
    define variable mchStoreType as character no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    molBufferList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = molBufferList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedBuffers
    then put stream stOutput unformatted "List of Shared Buffers ( Buffer name => Table Name )" skip "----------------------------------------------------" skip.
    else put stream stOutput unformatted "List of All Buffers ( Buffer name => Table Name )" skip "-------------------------------------------------" skip.
   
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).
      
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "BUFFER" then next.
      
      mobTableBuffer = cast( mobSymbol, TableBuffer ).
      mobTable = mobTableBuffer:getTable().
      minStoreType = mobTable:getStoretype().
      case minStoreType:
        when 1102 then mchStoreType = "DBTable".
        when 1103 then mchStoreType = "TempTable".
        when 1104 then mchStoreType = "WorkTable".
      end case.
      if mobSymbol:getDefineNode() <> ? then mlgDefined = yes. else mlgDefined = no.
      if mobSymbol:getIndirectDefineIdNode() <> ? then mlgIndirect = yes. else mlgIndirect = no.

      if mlgDefined
      then do:
        mlgIsNew = false.
        mlgIsGlobal = false.
        mobToken = mobSymbol:getDefineNode():firstChild().
        mchTokenType = mobToken:getText().
        if  mchTokenType = "NEW"
        then do:
          mlgIsNew = true.
          mobChildToken = mobToken:firstChild().
          if NodeTypes:getTypeName(mobChildToken:GetType()) = "GLOBAL"
          then do:
            mlgIsGlobal = true.
            mobChildToken = mobChildToken:nextSibling().
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
          end. /* if global */
          else do:
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
            then do:
              mobToken = mobToken:nextSibling().
              mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
            end.
          end. /* not global */
        end. /* new */
        else do:
          if mchTokenType = "SHARED" 
          then do:
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).
          end.
        end.         
        mchObjectType = NodeTypes:getTypeName(mobToken:GetType()).        
      end.

      if mchObjectType = "TEMPTABLE" or mchObjectType = "WORKTABLE" then next.

      if mlgDefined or mlgIndirect
      then do:
        if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedBuffers
        then do:
          put stream stOutput unformatted
              mobSymbol:fullName()
              " => "
              mobTable:getName()
              " ("
              mchStoreType
              ")"
              .
          if mlgDefined then put stream stOutput unformatted " from definition of " mchObjectType.
          if mlgIndirect then put stream stOutput unformatted " from indirect definition of " mchObjectType.
          if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
          if mobSymbol:isExported() and not mlgIsGlobal then put stream stOutput unformatted " - new shared ".
          if mobSymbol:isExported() and mlgIsGlobal then put stream stOutput unformatted " - new global shared ".
          put stream stOutput unformatted skip.
        end.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListBuffers */  
  
  method private void ListVariables(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedVariableList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobVariable as class Variable no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.
    define variable mlgAs as logical no-undo.
    define variable mlgLike as logical no-undo.
    define variable mobToken as JPNode no-undo.
    define variable mobChildToken as JPNode no-undo.
    define variable mchTokenType as character no-undo.
    define variable mlgIsNew as logical no-undo initial false.
    define variable mlgIsGlobal as logical no-undo initial false.
    define variable mchDataType as character no-undo.
    define variable mchName as character no-undo.
    define variable mchObjectType as character no-undo.
    
    if (ipobParseUnit eq ?) then return.  
    
    SharedVariableList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedVariableList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedVariables
    then put stream stOutput unformatted "List of Shared Variables" skip "------------------------" skip.
    else put stream stOutput unformatted "List of All Variables" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "VARIABLE" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobVariable = cast( mobSymbol, Variable ).
      if mobSymbol:getAsNode() <> ? then mlgAs = yes. else mlgAs = no.
      if mobSymbol:getLikeNode() <> ? then mlgLike = yes. else mlgLike = no.
      if mobSymbol:getDefineNode() <> ? then mlgDefined = yes. else mlgDefined = no.
      
      if mlgDefined
      then do:
        mlgIsNew = false.
        mlgIsGlobal = false.
        mobToken = mobSymbol:getDefineNode():firstChild().
        mchTokenType = mobToken:getText().
        if  mchTokenType = "NEW"
        then do:
          mlgIsNew = true.
          mobChildToken = mobToken:firstChild().
          if NodeTypes:getTypeName(mobChildToken:GetType()) = "GLOBAL"
          then do:
            mlgIsGlobal = true.
            mobChildToken = mobChildToken:nextSibling().
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
          end. /* if global */
          else do:
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
            then do:
              mobToken = mobToken:nextSibling().
              mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
            end.
          end. /* not global */
        end. /* new */
        else do:
          if mchTokenType = "SHARED" 
          then do:
            mobToken = mobToken:nextSibling().
            mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).
          end.
        end.         
        mchObjectType = NodeTypes:getTypeName(mobToken:GetType()).        
      end.

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedVariables
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mlgAs then put stream stOutput unformatted " as " mobVariable:getDataType():getProgressName().
        if mlgLike then put stream stOutput unformatted " like " mobSymbol:getLikeNode():firstChild():firstChild():getText() " ( " mobVariable:getDataType():getProgressName() " )".
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() and not mlgIsGlobal then put stream stOutput unformatted " - new shared ".
        if mobSymbol:isExported() and mlgIsGlobal then put stream stOutput unformatted " - new global shared ".
        if mobSymbol:getNumReads() > 0 then put stream stOutput unformatted " - assigned from".
        if mobSymbol:getNumWrites() > 0 then put stream stOutput unformatted " - assigned to".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListVariables */
  
  method private void ListFrames(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedFrameList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobFrame as class Frame no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedFrameList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedFrameList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedFrames
    then put stream stOutput unformatted "List of Shared Frames" skip "---------------------" skip.
    else put stream stOutput unformatted "List of All Frames" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "FRAME" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobFrame = cast( mobSymbol, Frame ).

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedFrames
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
        if mobFrame:isInitialized() 
        then put stream stOutput unformatted " - initialized".
        else put stream stOutput unformatted " - not initialized".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListFrames */

  method private void ListDatasets(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedDatasetList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobDataset as class Dataset no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedDatasetList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedDatasetList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedDatasets
    then put stream stOutput unformatted "List of Shared Datasets" skip "-----------------------" skip.
    else put stream stOutput unformatted "List of All Datasets" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "DATASET" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobDataset = cast( mobSymbol, Dataset ).

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedDatasets
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListDatasets */
  
  method private void ListMenus(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedMenuList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobMenu as class Menu no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedMenuList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedMenuList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedMenus
    then put stream stOutput unformatted "List of Shared Menus" skip "--------------------" skip.
    else put stream stOutput unformatted "List of All Menus" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "MENU" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobMenu = cast( mobSymbol, Menu ).

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedMenus
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListMenus */

  method private void ListStreams(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedStreamList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobStream as class Stream no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedStreamList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedStreamList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedStreams
    then put stream stOutput unformatted "List of Shared Streams" skip "----------------------" skip.
    else put stream stOutput unformatted "List of All Streams" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "STREAM" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobStream = cast( mobSymbol, Stream ).

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedStreams
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListStreams */

  method private void ListBrowses(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedBrowseList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobBrowse as class Browse no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedBrowseList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedBrowseList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedBrowses
    then put stream stOutput unformatted "List of Shared Browses" skip "----------------------" skip.
    else put stream stOutput unformatted "List of All Browses" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "BROWSE" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobBrowse = cast( mobSymbol, Browse ).

      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedBrowses
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
        if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListBrowses */

  method private void ListQueries(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedQueryList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable mobSymbolScope as class SymbolScope no-undo.
    define variable mobQuery as class Query no-undo.
    define variable mobToken as class JPNode no-undo.
    define variable mobChildToken as JPNode no-undo.
    define variable mchTokenType as character no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.
    define variable mlgIsDBTable as logical no-undo initial false.
    define variable mlgIsGlobal as logical no-undo initial false.
    define variable mlgIsShared as logical no-undo initial false.
    define variable mlgIsNew as logical no-undo initial false.
    define variable mlgIsPrivate as logical no-undo initial false.
    define variable mlgIsProtected as logical no-undo initial false.
    define variable mlgIsPublic as logical no-undo initial false.
    define variable mchQualifier as character no-undo.
    define variable mchFieldPhrase as character extent 10 no-undo.
    define variable mchQueryName as character no-undo.
    define variable mchRecordName as character extent 10 no-undo.
    define variable minNumRecords as integer no-undo.
    define variable mchTableName as character extent 10 no-undo.
    define variable mchAdditionalInfo as character no-undo.
    define variable minStoreType as integer no-undo.
    define variable mchStoreType as character extent 10 no-undo.
    define variable mobBufferSymbol as class Symbol no-undo.
    define variable mobTableBuffer as class TableBuffer no-undo.
    define variable mobTable as class org.prorefactor.core.schema.Table no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedQueryList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedQueryList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    if not lgShowNonSharedQueries
    then put stream stOutput unformatted "List of Shared Queries" skip "----------------------" skip.
    else put stream stOutput unformatted "List of All Queries" skip "-------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) <> "QUERY" then next.
      mobSymbolScope = mobSymbol:getScope().
      mobQuery = cast( mobSymbol, Query ).
      mobToken = mobSymbol:getDefineNode():firstChild().
      mchTokenType = NodeTypes:getTypeName(mobToken:GetType()).      
      mchQualifier = mchTokenType.
      case mchTokenType:
        when "NEW"
          then do:
            mlgIsNew = true.
            mchQualifier = "NEW".
            mobChildToken = mobToken:firstChild().
            if NodeTypes:getTypeName(mobChildToken:GetType()) = "GLOBAL"
            then do:
              mlgIsGlobal = true.
              mchQualifier = mchQualifier + " GLOBAL".
              mobChildToken = mobChildToken:nextSibling().
              if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
              then mlgIsShared = true.
            end. /* if global */
            else do:
              if NodeTypes:getTypeName(mobChildToken:GetType()) = "SHARED"
              then mlgIsShared.
            end. /* not global */
            mchQualifier = mchQualifier + " SHARED".
          end. /* new */
        when "SHARED"
          then do:
            mlgIsShared = true.
          end.         
        when "PRIVATE"
          then do:
            mlgIsPrivate = true.
          end.         
        when "PROTECTED"
          then do:
            mlgIsProtected = true.
          end.         
        when "PUBLIC"
          then do:
            mlgIsPublic = true.
          end.         
        otherwise
          do:
            put stream stOutput unformatted substitute( "SharedInfoLister: Confused parsing got &1 for scoping qualifier", mchTokenType ) skip.
          end.
      end case.
      
      if not ( mlgIsShared or mlgIsProtected or mlgIsPublic ) then return.
      mobToken = mobToken:nextSibling().  /* QUERY */
      mobToken = mobToken:nextSibling().  /* FOR */
      mobToken = mobToken:nextSibling().  /* ID */
      mchQueryName = mobToken:getText().  
      mobToken = mobToken:nextSibling().  /* FOR*/  
      minNumRecords = 1.

      RECORD_NAME_BLOCK:
      repeat:
        mchRecordName[ minNumRecords ] = mobToken:getText().
        mobBufferSymbol = mobToken:getSymbol().
        mobTableBuffer = cast( mobBufferSymbol, TableBuffer ).
        mobTable = mobTableBuffer:getTable().
        mchTableName[ minNumRecords ] = mobTable:getName().
        minStoreType = mobTable:getStoretype().
        case minStoreType:
          when 1102 then mchStoreType[ minNumRecords ] = "DBTable".
          when 1103 then mchStoreType[ minNumRecords ] = "TempTable".
          when 1104 then mchStoreType[ minNumRecords ] = "WorkTable".
        end case.
        if NodeTypes:getTypeName(mobToken:nextSibling():GetType()) = "FIELDS"
        then do:
          mobToken = mobToken:nextSibling().  /* FIELDS */
          mchFieldPhrase[ minNumRecords ] = RestOfPhrase( mobToken:firstChild() ).
        end.
        if NodeTypes:getTypeName(mobToken:nextSibling():GetType()) = "COMMA"
        then do:
          mobToken = mobToken:nextSibling().  /* COMMA */
          mobToken = mobToken:nextSibling().  /* NEXT RECORD_NAME */
          minNumRecords = minNumRecords + 1.
          next RECORD_NAME_BLOCK.
        end.
        else do:
          mobToken = mobToken:nextSibling().
          leave.
        end.
      end.

      mchAdditionalInfo = RestOfPhrase( mobToken ).
      if ( mobSymbol:isImported() or mobSymbol:isExported() ) or lgShowNonSharedQueries
      then do:
        put stream stOutput unformatted mobSymbol:fullName().
        put stream stOutput unformatted " [ " mchQualifier " ] for ".
        do minWhich = 1 to minNumRecords:
          if minWhich > 1 then put stream stOutput unformatted "; " skip "    ".
          put stream stOutput unformatted mchTableName[ minWhich ] ", a " mchStoreType[ minWhich ] " with buffer " mchRecordName[ minWhich ].
          if mchFieldPhrase[ minWhich ] <> "" then put stream stOutput unformatted " " mchFieldPhrase[ minWhich ].
        end.
        if mchAdditionalInfo <> "" then put stream stOutput unformatted " - " mchAdditionalInfo.
        put stream stOutput unformatted skip.
      end.
    end.
    
    catch eobSysError as Progress.Lang.SysError :
      put stream stOutput unformatted substitute("Unexpected Exception: &3", eobSysError:getMessage(1)) skip.
      if eobSysError:NumMessages > 1
      then do minWhich = 2 to eobSysError:NumMessages:
        put stream stOutput unformatted substitute("Unexpected Exception (cont.): &1", eobSysError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobSysError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobSysError:CallStack,"~n") skip.
      end.
      delete object eobSysError no-error.
    end catch.
              
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListQueries */

  method private void ListRest(
      ipobParseUnit as class ParseUnit
      ): 
    define variable SharedVariableList as class java.util.ArrayList no-undo.
    define variable mobIterator as class java.util.Iterator no-undo.
    define variable mobSymbol as class Symbol no-undo.
    define variable minWhich as integer no-undo.
    define variable mlgDefined as logical no-undo.
    define variable mlgIndirect as logical no-undo.

    if (ipobParseUnit eq ?) then return.  
    
    SharedVariableList = ipobParseUnit:getRootScope():getAllSymbolsDeep( ).
    mobIterator = SharedVariableList:iterator().
    
    put stream stOutput unformatted "" skip(1).
    put stream stOutput unformatted "List of Other Things" skip "--------------------" skip.
    
    do while mobIterator:hasNext():
      mobSymbol = cast( mobIterator:next(), Symbol ).

      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "Program_Root" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "BUFFER" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "VARIABLE" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "FRAME" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "DATASET" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "MENU" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "SUBMENU" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "MENUITEM" then next.
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "STREAM" then next.        
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "BROWSE" then next.   
      if NodeTypes:getTypeName(mobSymbol:getProgressType()) = "QUERY" then next.   
            
      put stream stOutput unformatted NodeTypes:getTypeName(mobSymbol:getProgressType()) "| ".
      put stream stOutput unformatted mobSymbol:fullName().
      if mobSymbol:isImported() then put stream stOutput unformatted " - shared ".
      if mobSymbol:isExported() then put stream stOutput unformatted " - new shared ".
      put stream stOutput unformatted skip.
    end.
    
    catch eobOtherError as Progress.Lang.Error :
      put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
      if eobOtherError:NumMessages > 1
      then do minWhich = 2 to eobOtherError:NumMessages:
        put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(minWhich)) skip.
      end.
      do minWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
        put stream stOutput unformatted entry(minWhich,eobOtherError:CallStack,"~n") skip.
      end.
      delete object eobOtherError no-error.
    end catch.
     
  end method. /* ListRest */    

  method private character RestOfPhrase(
      ipobNode as JPNode
      ):
    define variable mchRestOfPhrase as character initial "" no-undo.

    if ipobNode = ? then return mchRestOfPhrase.
    if NodeTypes:getTypeName(ipobNode:getType()) = "PERIOD" then return mchRestOfPhrase.
    if NodeTypes:getTypeName(ipobNode:getType()) = "COMMA" then return mchRestOfPhrase.
    
    mchRestOfPhrase = ipobNode:getText().
    if ipobNode:firstChild() <> ? and NodeTypes:getTypeName(ipobNode:firstChild():getType()) <> "PERIOD"
    then mchRestOfPhrase = mchRestOfPhrase + " " + RestOfPhrase( ipobNode:firstChild() ).
    if ipobNode:nextSibling() <> ? and NodeTypes:getTypeName(ipobNode:nextSibling():getType()) <> "PERIOD"
    then mchRestOfPhrase = mchRestOfPhrase + " " + RestOfPhrase( ipobNode:nextSibling() ).
    return mchRestOfPhrase.
  end method.

end class.

