/*------------------------------------------------------------------------
    File        : TestScanner
    Purpose     : 
    Syntax      : 
    Description : Controls the processing of the ABL2DB components.
    Author(s)   : Thomas Mercer-Hursh
    Created     : Sat Mar 22 10:41:43 CDT 2014
    Notes       : Use Process for stepwise progression of all standard functions
                : Use ProcessX individually for special purposes
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.ABL2DB.BuildDirectoryTree.
using com.cintegrity.Proparse.DocSchemaFromDB.
using com.cintegrity.Proparse.ScanOneCU.
using com.cintegrity.Proparse.SetEnvironment.
using com.cintegrity.Proparse.SetSchemaFromFile.

routine-level on error undo, throw.

class com.cintegrity.Proparse.TestScanner inherits BuildDirectoryTree: 
  define public property chDataBaseList as character no-undo get. set.
  define public property chLogicalDataBaseList as character no-undo get. set.
  define public property chAliasList as character no-undo get. set.
  define public property chSchemaDirectory as character no-undo get. set.
  define public property chSchemaDumpFile as character no-undo get. set.
  define public property chSchemaFileList as character no-undo get. set.
  define public property chCompilableExtensions as character no-undo get. set.
  define public property chSourceExtensions as character no-undo get. set.
  define public property chXrefDirectoryList as character no-undo get. set.
  define public property lgShowErrorDetails as logical no-undo initial false get. set.
  
  define private variable obDocSchemaFromDB as class DocSchemaFromDB no-undo.

	constructor public TestScanner (  
      ipchSourceDirectoryList as character
      ):  
    super ( ipchSourceDirectoryList ).
    		
	end constructor.

	destructor public TestScanner ( ):
    obReport:WriteMessage ( substitute( "TestScanner compile exceptions end - &1", now ) ).
	end destructor.

  method public override void Initialize():
    define variable minWhich as integer no-undo.
    
    super:Initialize().

    do minWhich = 1 to num-entries( chSourceDirectoryList ):
    
      file-info:file-name = entry( minWhich, chXrefDirectoryList).
      if file-info:full-pathname = ? 
          or not file-info:file-type begins "D" 
          or not file-info:file-type matches "*W*"
      then do:
        log-manager:write-message (substitute("CompileDirectoryTree ERROR: XREF Path '&1' not found or not writeable", entry( minWhich, chXrefDirectoryList) )).
        undo, throw new Progress.Lang.AppError( "CompileDirectoryTree: Unable to initialize." ).  
      end.     

    end. /* do minWhich */ 
    obReport:WriteMessage ( substitute( "TestScanner compile exceptions - &1", now ) ).
  end method.
  
  method public override void Process ( ):
   /* set chValidExtensions and/or chExcludeExtensions and/or lgExcludeNoExtension as necessary in launcher */
    super:Process().

    ScanAll().
  end method.
  
  method private void ScanAll():
    define buffer bfContents for ttContents.

    for each bfContents where not bfContents.lgIsProcessed and not lgIsDir:
      ScanOne( 
          bfContents.chSourceDirectory,
          bfContents.chName,
          bfContents.chFullPath,
          bfContents.chPackage,
          bfContents.chExtension
          ).
      bfContents.lgIsProcessed = true.
    end. /* for each */   
    
  end method.
  
  method private void ScanOne(
       ipchSourceDirectory as character,
      ipchName as character,
      ipchFullPath as character,
      ipchPackage as character,
      ipchExtension as character
      ):
    define variable mchXrefTarget as character no-undo.
    define variable mchXrefFile as character no-undo.
    define variable minError as integer no-undo.
    define variable minWhich as integer no-undo.
    define variable mchType as character no-undo initial "Error,Warning".
    define variable obScanOneCU as class ScanOneCU no-undo.
    
    minWhich = lookup( ipchSourceDirectory, chSourceDirectoryList ).
    if ipchPackage > ""
    then mchXrefTarget = entry( minWhich, chXrefDirectoryList ) + chSeparator + ipchPackage.
    else mchXrefTarget = entry( minWhich, chXrefDirectoryList ).
    mchXrefFile = obPathUtilities:XrefFile( entry( minWhich, chXrefDirectoryList ), ipchPackage, ipchName ).
    
    do on error undo, throw:
      obPathUtilities:MakeDirectory (mchXrefTarget).
      catch eobAppError as Progress.Lang.AppError:
        undo, throw new Progress.Lang.AppError( "Aborting TestScanner - problem with MakeDirectory" ).
      end catch.
    end. 
        
    if ipchExtension = "cls"
    then do:
      compile value(ipchFullPath)
          xref-xml value(mchXrefTarget)
          v6frame use-revvideo
          no-error.
    end.
    else do:
      compile value(ipchFullPath)
          xref-xml value(mchXrefTarget)
          v6frame use-revvideo
          no-error.
    end.    

    if compiler:error 
    then do:
      obReport:WriteMessage ( substitute("TestScanner: Compile error for &1", ipchFullPath ) ).
      if compiler:num-messages > 0 and lgShowErrorDetails
      then do minError = 1 to compiler:num-messages:
        if compiler:get-message-type(minError) = 1 
        then obReport:WriteMessage (  substitute("TestScanner: &1: &2", entry(compiler:get-message-type(minError), mchType), compiler:get-message(minError)) ).
      end.
      return.      
    end.
    file-info:file-name = mchXrefFile.
    if file-info:full-pathname = ? 
    then do:
      obReport:WriteMessage ( substitute("TestScanner: No XREF file (&1) created from Compile of &2", mchXrefFile, ipchFullPath ) ).
      return.
    end.
  
    obScanOneCU = new ScanOneCU( ipchFullPath ).
    obScanOneCU:chSchemaDirectory = chSchemaDirectory.           
    obScanOneCU:chSchemaDumpFile = chSchemaDumpFile.
    obScanOneCU:chLogicalDataBaseList = chLogicalDataBaseList.
    obScanOneCU:chAliasList = chAliasList.
    
    obScanOneCU:Initialize().
    obScanOneCU:DoParse().
    
    delete object obScanOneCU.

  end method.
  
end class.