/*------------------------------------------------------------------------
    File        : tokenlist_launcher.p
    Purpose     : 
    Syntax      : 
    Description : Launch program for TokenLister
    Author(s)   : Thomas Mercer-Hursh
    Created     : Mon Mar 05 14:53:16 PST 2012
    Usage       : Use DocSchemaFromDB or DocSchemaFromDF to create proparse.schema
                : Also created by BuildSchemaDF in ABL2DB
    Notes       : chSchemaDirectory    Directory in which proparse.schema resides
                : chSchemaDumpFile     Name of proparse.schema file if not default 
                : chDataBaseList       List of Database names (only required for aliases)
                : chAliasList          List of Alias names (if any)
                : lgDeletePPSchemaFile Flag of whether to delete proparse.schema when done (default false)
  ----------------------------------------------------------------------*/

using Progress.Lang.*.
using com.cintegrity.Proparse.TokenLister.
/* Uncomment one or neither as needed 
using com.cintegrity.Proparse.DocSchemaFromDB.
using com.cintegrity.Proparse.DocSchemaFromDF.
*/

block-level on error undo, throw.

/* Set the following as desired to parse the desired file */
define variable chSchemaDirectory as character no-undo initial "C:\work\IS\DB".
define variable chSchemaDumpFile as character no-undo initial "proparse.schema".
define variable chLogicalDataBaseList as character no-undo initial "c:\work\IS\DB\ISRel,c:\work\IS\DB\ISSort".
define variable chAliasList as character no-undo initial "".
define variable chFileToParse as character no-undo initial "c:\work\ABL2UML\ABL2DB\src\com\cintegrity\Proparse\TestCases\TT_WF_Buffer.p".
define variable chOutputFile as character no-undo initial "c:\tmp\tokenlistoutput.txt".

define variable inWhich as integer no-undo.

define variable obTokenLister as class TokenLister no-undo.
/* Uncomment one or neither as needed 
define variable obDocSchemaFromDB as class DocSchemaFromDB no-undo.
define variable obDocSchemaFromDF as class DocSchemaFromDF no-undo.
*/
define stream stOutput.

do on error undo, leave:
  /* Sample use of DocSchemaFromFile
  obDocSchemaFromDB = new DocSchemaFromDB().
  obDocSchemaFromDB:chSchemaDirectory. 
  obDocSchemaFromDB:chSchemaDumpFile.
  obDocSchemaFromDB:chDataBaseList. 
  obDocSchemaFromDB:Initialize().
  obDocSchemaFromDB:DumpSchemas().
  */
  
  obTokenLister = new TokenLister().
  obTokenLister:chSchemaDirectory = chSchemaDirectory.
  obTokenLister:chSchemaDumpFile = chSchemaDumpFile.
  obTokenLister:chLogicalDataBaseList = chLogicalDataBaseList.
  obTokenLister:chAliasList = chAliasList.
  obTokenLister:chFileToParse = chFileToParse.
  obTokenLister:chOutputFile = chOutputFile.
  obTokenLister:inTabSize = 4.
  obTokenLister:lgShowColumn = yes.
  obTokenLister:lgShowProparseDirectives = no.
  obTokenLister:lgShowSourceFileName = no.
  obTokenLister:lgShowSourceLineNo = yes.
  obTokenLister:lgShowSecondaryKeyword = no.
  obTokenLister:lgShowStatementHeadFlag = no.
  obTokenLister:lgShowTableType = yes.
  obTokenLister:lgShowSymbols = yes.
  
  obTokenLister:Initialize().
  obTokenLister:DoList().
  catch eobAppError as Progress.Lang.AppError:
    output stream stOutput to value(chOutputFile).
    put stream stOutput unformatted substitute("AppError: &1", eobAppError:ReturnValue ) skip.  
    delete object eobAppError.
    output stream stOutput close.
  end catch.
  
  catch eobOtherError as Progress.Lang.Error :
    output stream stOutput to value(chOutputFile).
    put stream stOutput unformatted substitute("Error: &1", eobOtherError:getMessage(1)) skip.  
    if eobOtherError:NumMessages > 1
    then do inWhich = 2 to eobOtherError:NumMessages:
      put stream stOutput unformatted substitute("Error (cont.): &1", eobOtherError:getMessage(inWhich)) skip.
    end.
    do inWhich = 1 to num-entries(eobOtherError:CallStack,"~n"):
      put stream stOutput unformatted entry(inWhich,eobOtherError:CallStack,"~n") skip.
    end.
    delete object eobOtherError no-error.
    output stream stOutput close.
  end catch.
end. /* do */